package uibk.applets.biegelinie;

import java.awt.Color;
import java.awt.GridLayout;

import uibk.mtk.draw2d.axes2d.Axes2D;
import uibk.mtk.draw2d.objects.Graph2D;
import uibk.mtk.draw2d.objects.MathArrow2D;
import uibk.mtk.draw2d.objects.MathLine2D;
import uibk.mtk.draw2d.objects.PhysicBaseFix2D;
import uibk.mtk.draw2d.objects.PhysicBaseMove2D;
import uibk.mtk.draw2d.objects.PhysicBaseWall;
import uibk.mtk.lang.Messages;
import uibk.mtk.math.functions.Function1D;
import uibk.mtk.math.parsing.Function1DParser;
import uibk.mtk.swing.appletbase.AppletBase;
import uibk.mtk.swing.base.MPanel;
import uibk.mtk.swing.scene2d.PanelScale2D;
import uibk.mtk.draw2d.base.MathPanel2D;

/**
 * Applet fr Biegelinie/reine Biegung
 * 
 * <p>
 * Mit diesem Applet kann die Biegelinie je nach Beanspruchung des Trgers
 * visualisiert werden.
 * </p>
 * 
 * <p>
 * Fr eine Beschreibung des Applets siehe <a
 * href="http://www.mathe-online.at/nml/materialien/innsbruck/kurven2d/">www.mathe-online.at/nml/materialien/innsbruck/kurven2d/
 * </a>
 * </p>
 * 
 * <p>
 * Copyright: Institut fr Technische Mathematik, Geometrie und Bauinformatik,
 * 2006
 * </p>
 * 
 * @author Rudigier Andreas (csa7164@uibk.ac.at)
 */
public class AppletBiegelinie extends AppletBase {

	/** Art der Belastung */
	int art=1;
	
	/** Art der Lagerung */
	int lager=1;
	
	/** Krfte */
	MathArrow2D one[] = new MathArrow2D[11];

	/** Funktion der Streckenlast Gerade */
	MathLine2D line;

	/** Graph der Funktion der sinusfrmigen Streckenlast */
	Graph2D func;

	/** Funktion der sinusfrmigen Streckenlast */
	Function1D function;
	
	/** Lngeverhltnis*/
	double q=1;
	
	/** Lnge des Balkens */
	double length=100;
	
	/** Hhe der Kraft */
	double high=0;

	/** Kragarm */
	PhysicBaseWall krag = new PhysicBaseWall();
	
	/** Festlager */
	PhysicBaseFix2D fest = new PhysicBaseFix2D();

	/** Loslager */
	PhysicBaseMove2D los = new PhysicBaseMove2D();

	/** Balken */
	MathLine2D balken = new MathLine2D();

	/** Array der Mathpanels* */
	MathPanel2D[] mathpanel = new MathPanel2D[6];

	/** Die Achsen */
	Axes2D axes[] = new Axes2D[5];

	/** Der Graph der Kurven */
	Graph2D graph[] = new Graph2D[5];

	/** Panel zum Skalieren der Achsen */
	PanelScale2D panelaxes;

	/** Verknpfung zur Berechnung der Krfte */
	Power power;
	
	/** Panel zum definieren der Kurve */
	PanelDef paneldef;

	/** Variable in der die Kurve def. sein muss */
	static final char VAR = 't';

	/** Farbe fr die Kraft */
	static final Color COLORZWEIBEIN = Color.red;

	/** Anzahl der standardmig zum Plotten verwendeten Punkte */
	static final int DEFAULTNUMPOINTS = 500;

	/** Name des Resourcebundles */
	static final String BUNDLE_NAME = "uibk.applets.biegelinie.messages";//$NON-NLS-1$

	/** Initialisierung der Komponenten */
	protected void initComponents() {
	
		super.initComponents();
		super.panelnav.enableTheoryButton(false);
		super.panelnav.setStrcopyright(Messages.getString(BUNDLE_NAME,
				"AppletBiegelinie.3"));
		super.panelnav.setStrauthor(Messages.getString(BUNDLE_NAME,
		"AppletBiegelinie.4"));
		super.panelnav.setHome("../indexAnd.html");
		/** Der <CODE>MathPanel2D</CODE> fr die Streckenlast */
		mathpanel[0] = new MathPanel2D();

		/** Der <CODE>MathPanel2D</CODE> fr die Querkraft */
		mathpanel[1] = new MathPanel2D();

		/** Der <CODE>MathPanel2D</CODE> fr das Moment */
		mathpanel[2] = new MathPanel2D();

		/** Der <CODE>MathPanel2D</CODE> fr die Hilfsvariable */
		mathpanel[3] = new MathPanel2D();

		/** Der <CODE>MathPanel2D</CODE> fr die Durchbiegung */
		mathpanel[4] = new MathPanel2D();

		/** Der <CODE>MathPanel2D</CODE> fr die Darstellung der Belastungsart */
		mathpanel[5] = new MathPanel2D();
		
		//Drag and drop einschalten
		mathpanel[5].setListenForDragAndDrop(true);
		
		//Referenz auf die Visualisierungssteurung
		power=new Power(this);
		
		for (int i = 0; i < 5; i++) {
			axes[i] = new Axes2D();
			axes[i].setCaptionXAxis("x"); //$NON-NLS-1$
			axes[i].setStyleYAxis(1);
			}

		axes[0].setCaptionYAxis("p(x)"); //$NON-NLS-1$
		axes[1].setCaptionYAxis("Q(x)"); //$NON-NLS-1$
		axes[2].setCaptionYAxis("M(x)"); //$NON-NLS-1$
		axes[3].setCaptionYAxis("\u03A8(x)"); //$NON-NLS-1$
		axes[3].setDirectionYAxis(2);
		axes[4].setCaptionYAxis("w(x)"); //$NON-NLS-1$
		axes[4].setDirectionYAxis(2);

		for (int i = 0; i < 5; i++) {
			graph[i] = new Graph2D();
			mathpanel[i].add(axes[i]);
			mathpanel[i].add(graph[i]);
			graph[i].setVisible(false);
			mathpanel[i].getScene2d().enableRoundX(true);
			mathpanel[i].getScene2d().enableRoundY(true);
		}
		// zeichnet den Balken
		mathpanel[5].getScene2d().enableRoundX(false);
		mathpanel[5].getScene2d().enableRoundY(false);
		mathpanel[5].setDoubleBuffered(true);
		mathpanel[5].setToolTipText("");
		fest.setOrientation(PhysicBaseFix2D.NORTH);
		fest.setColor(Color.black);
		mathpanel[5].add(fest);
		fest.setCoordinates(0, 0);
		fest.setSize(40);
		los.setOrientation(PhysicBaseFix2D.NORTH);
		los.setColor(Color.black);
		mathpanel[5].add(los);
		los.setCoordinates(length, 0);
		los.setSize(40);
		krag.setOrientation(PhysicBaseWall.SOUTH);
		krag.setColor(Color.BLACK);
		krag.setCoordinates(length,0);
		mathpanel[5].add(krag);
		krag.setSize(40);
		power.drawbalken(2);

		// Krfte und Streckenlasten
		power.setMathPanel2D(mathpanel[5]);
		line = new MathLine2D();
		mathpanel[5].add(line);

		func = new Graph2D();
		func.setColor(Color.BLACK);
		mathpanel[5].add(func);

		try {
			function = Function1DParser.parse("x", new String("50*sin(x*"
					+ Double.toString(Math.PI/length) + ")"));
		} catch (Exception e) {
		}

		for (int i = 0; i < 11; i++) {
			one[i] = new MathArrow2D();
			mathpanel[5].add(one[i]);
		}

		// Panel mit dem Urbild und dem Bild erstellen
		MPanel main1 = new MPanel();
		main1.setLayout(new GridLayout(2, 2));

		for (int i = 0; i < 4; i++) {
			main1.add(mathpanel[i]);
		}

		MPanel main2 = new MPanel();
		main2.setLayout(new GridLayout(1, 2));
		main2.add(mathpanel[4]);
		main2.add(mathpanel[5]);

		MPanel main = new MPanel();
		main.setLayout(new GridLayout(2, 1));
		main.add(main1);
		main.add(main2);

		super.setMainPanel(main);
		super.setControlPanel(new PanelCommand(this));

	}

	public static void main(String[] args) {
		runmain(new AppletBiegelinie(), Messages.getString(BUNDLE_NAME,
				"AppletBiegelinie.2")); //$NON-NLS-1$
	}

}