package uibk.applets.biegelinie;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.ComponentEvent;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.util.Vector;
import java.text.NumberFormat;

import uibk.mtk.math.functions.ParsedFunction1D;
import uibk.mtk.draw2d.base.Drawable2D;
import uibk.mtk.draw2d.base.MathPanel2D;
import uibk.mtk.draw2d.objects.MathArrow2D;
import uibk.mtk.draw2d.objects.DragablePoint2D;
import uibk.mtk.draw2d.util.GraphUtil;
import uibk.mtk.geom.geom2d.Punkt2D;
import uibk.mtk.lang.Dragable;
import uibk.mtk.lang.PrepaintComputable;
import uibk.mtk.math.parsing.Function1DParser;


/**
 * Visualisierung der Biegelinie und den einzelnen Integrationsschritten der DGL
 * <p>
 * F?r eine Beschreibung des Applets siehe <a
 * href="http://www.mathe-online.at/nml/materialien/innsbruck/integration2d/">www.mathe-online.at/nml/materialien/innsbruck/integration2d/
 * </a>
 * </p>
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2006
 * </p>
 * 
 * @author Rudigier Andreas (csae7164@uibk.ac.at)
 */
public class Power extends Drawable2D implements PrepaintComputable, Dragable {

	/** Verknpfung zur main-Klasse */
	AppletBiegelinie main;

	/** Radius der Punkte, die die Intervallgrenzen visualisieren */
	int pointradius = 3;

	/** aktuell mit Maus verschobene Intervallgrenze */
	Dragable dragged = null;

	/** Vektor mit den Punkten die die Intervallgrenzen markieren */
	Vector points = new Vector();

	/** Stufe der Integration */
	int s = 6;


	/** Punkte fr die Darstellung */
	Punkt2D[] punkt = new Punkt2D[1];

	/** Kraftpfeile */
	MathArrow2D[] power = new MathArrow2D[3];

	/** Verschiebarer zweiter Punkt der Streckenlast */
	MoveX last = new MoveX(0);

	/** Verschiebarer Punkt fr die Hhe Streckenlast */
	MoveY high = new MoveY(0);

	/** Formatieren der Ausgabe des ToolTips */
	NumberFormat nf = NumberFormat.getInstance();

	/** Erzeugt die Visualisierung */
	public Power(AppletBiegelinie main) {
		this.main = main;
		last.enableToolTip(true);
		high.enableToolTip(true);
		for (int i = 0; i < 3; i++) {
			power[i] = new MathArrow2D();
			main.mathpanel[0].add(power[i]);
		}
		power[0].setHead(new Punkt2D(0, 0));
		power[0].setVisible(true);
		power[1].setVisible(false);
		power[2].setVisible(true);
		power[0].setStroke(new BasicStroke(5));
		power[1].setStroke(new BasicStroke(5));
		power[2].setStroke(new BasicStroke(5));
		power[0].setColor(Color.BLUE);
		power[1].setColor(Color.BLACK);
		power[2].setColor(Color.BLUE);
		nf.setMaximumFractionDigits(4);
	}

	/**
	 * Gibt zur?ck ob beim MouseEvent <CODE>ev</CODE> nur der linke Knopf
	 * gedr?ckt worden sit
	 */
	private boolean onlyLeftButton(MouseEvent ev) {
		return (ev.getModifiers() == InputEvent.BUTTON1_MASK);
	}

	/** Klasse fr die Punkte der Krfte */
	class MoveY extends DragablePoint2D {

		public MoveY(double x) {
			super(0, x);
		}

		/**
		 * Gibt den Tooltip des Punktes zur?ck, falls keiner gesetzt wurde,d.h.
		 * falls toolTip==null, wird "(x,y)" zur?ckgegeben
		 */
		public String getToolTipText(MouseEvent ev) {

			if (this.toolTip == null) {
				String str = "(" //$NON-NLS-1$
						+ nf.format(x / 100) + "," //$NON-NLS-1$
						+ nf.format(y / 100) + ")"; //$NON-NLS-1$
				return (str);
			} else {
				return toolTip;
			}
		}

		/** Ziehen des ausgew?hlten Punktes */
		public void drag(MouseEvent ev) {
			double t = scene2d.pixelToY(ev.getY());
			double u = scene2d.pixelToX(ev.getX());
			if (t < 10 || t > 100)
				return;
			y = t;
			main.high = t;
			deletepower();
			this.getToolTipText(ev);
			switch (main.art) {
			case 1:
				if (u < 0 || u > main.length) {
					if (u < 0) {
						draw1(0, t);
					} else {
						draw1(main.length, t);
					}
				} else {
					draw1(u, t);
				}
				break;
			case 2:
				if (main.lager == 2) {
					draw2(0, last.getX(), t);
				} else {
					draw2(main.length, last.getX(), t);
				}
				break;
			case 3:
				if (main.lager == 2) {
					draw3(0, main.length, t);
				} else {
					draw3(main.length, 0, t);
				}
				break;
			case 4:
				draw4(t);
				break;
			default:
				break;
			}
			compute();
		}
	}

	/** Klasse fr die Punkte der Krfte */
	class MoveX extends DragablePoint2D {

		public MoveX(double x) {
			super(x, 0);
		}

		/**
		 * Gibt den Tooltip des Punktes zurck.
		 */
		public String getToolTipText(MouseEvent ev) {

			String str = "(" //$NON-NLS-1$
					+ nf.format(x / 100) + "," //$NON-NLS-1$
					+ nf.format(y / 100) + ")"; //$NON-NLS-1$
			return (str);
		}

		/** Ziehen des ausgew?hlten Punktes */
		public void drag(MouseEvent ev) {
			double a, b;
			double t = scene2d.pixelToX(ev.getX());
			if (t < 0 || t > main.length)
				return;
			x = t;
			deletepower();
			switch (main.art) {
			case 1:
				draw1(x, main.high);
				break;
			case 2:
				b = 0;
				if (main.lager == 2) {
					a = 0;
					if (this.x == last.x) {
						b = last.x < 10 ? 10 : x;
						x = b;
					}
				} else {
					a = main.length;
					if (this.x == last.x) {
						b = last.x > 0.9 * main.length ? 0.9 * main.length : x;
						x = b;
					}

				}
				draw2(a, b, main.high);
				break;
			case 3:
				/*
				 * if (this.x == first.x) { b = last.x; a = first.x > 90 ? 90 :
				 * x; if (first.x + 10 > last.x) b = a + 10; last.x = b; x = a; }
				 * else { a = first.x; b = last.x < 10 ? 10 : x; if (first.x +
				 * 10 > last.x) a = b - 10; first.x = a; x = b; }
				 */
				if (main.lager == 2) {
					draw3(0, main.length, main.high);
				} else {
					draw3(main.length, 0, main.high);
				}
				break;

			default:
				break;
			}
			compute();
		}
	}

	/** Ziehen des ausgew?hlten Punktes */
	public void drag(MouseEvent ev) {
		if (!onlyLeftButton(ev))
			return;
		dragged.drag(ev);
	}

	/**
	 * Gibt zur?ck, ob die Maus im "Ziehbereich" eines der Punkte ist
	 */
	public boolean dragcontains(int xm, int ym) {
		if (last.dragcontains(xm, ym)) {
			dragged = last;
			return true;
		}
		if (high.dragcontains(xm, ym)) {
			dragged = high;
			return true;
		}

		return false;
	}

	/** Ziehen aktivieren/deaktivieren */
	public void enabledrag(boolean enable) {

	}

	/** Setzen des Mathpanel */
	public void setMathPanel2D(MathPanel2D panel) {
		super.setMathPanel2D(panel);
	}

	Vector lines = new Vector();

	/** Berechnung der Komponenten der Visualiserung */
	public void prepaintcompute() throws Exception {

	}

	/** Zeichnen der Visualisierung */
	public void draw(BufferedImage framebuffer, Graphics2D g2) {

	}

	/** Erzeugt die Visualisierung mit der aktuellen Einstellungen */
	public void create() throws Exception {

	}

	/** Gibt zur?ck ob das Drag & Drop aktiviert ist. */
	public boolean isDragEnabled() {
		return true;
	}

	/** leere Implementierung */
	public void componentShown(ComponentEvent ev) {
	}

	/** leere Implementierung */
	public void componentHidden(ComponentEvent ev) {
	}

	/** leere Implementierung */
	public void componentMoved(ComponentEvent ev) {
	}

	/** Neu Berechnen und Zeichnen veranlassen */
	public void componentResized(ComponentEvent ev) {
	}

	/**
	 * Lscht das Mathpanel und zeichnet den Biegebalken
	 */
	void deletepower() {
		for (int i = 0; i < 11; i++)
			main.one[i].setVisible(false);
		last.setVisible(false);
		high.setVisible(false);
		main.line.setVisible(false);
		main.func.setVisible(false);

	}

	/**
	 * Zeichnet je nach Lastfall
	 */
	void draw() {
		deletepower();
		switch (main.art) {
		case 1:
			if (main.length < high.getX() / main.q)
				high.setX(main.length);
			if (high.getY() == 0)
				break;
				draw1(high.getX() / main.q, high.getY());
			main.q = 1;
			break;
		case 2:
			if (main.length < last.getX() / main.q)
				last.setX(main.length);
			if (main.lager==2) {
				draw2(0, last.getX() / main.q, high.getY());
			} else {
				draw2(main.length, last.getX() / main.q, high.getY());
			}
			main.q = 1;
			break;
		case 3:
			/*
			 * if (main.length < last.getX()) last.setX(main.length); if
			 * (main.length < first.getX()) first.setX(main.length - 10);
			 */
			if (main.lager==2) {
				draw3(0, main.length, high.getY());
			} else {
				draw3(main.length,0, high.getY());
			}
			break;
		case 4:
			draw4(high.getY());
			break;

		default:
			break;
		}
	}

	/**
	 * Zeichnet den Balken
	 */
	void drawbalken(int i) {
		main.mathpanel[5].getScene2d().setLimits(-main.length / 20,
				21 / 20 * main.length, -50, 100);
		if (i == 2) {
			main.fest.setVisible(true);
			main.los.setVisible(true);
			main.los.setCoordinates(main.length, 0);
			main.krag.setVisible(false);
			main.balken.setP1(main.los.getPunkt2D());
			main.balken.setP2(main.fest.getPunkt2D());
		} else {
			main.fest.setVisible(false);
			main.los.setVisible(false);
			main.krag.setVisible(true);
			main.krag.setCoordinates(0, 0);
			main.balken.setP1(main.krag.getPunkt2D());
			main.balken.setP2(new Punkt2D(main.length, 0));
		}

		main.balken.setStroke(new BasicStroke(2));
		main.mathpanel[5].add(main.balken);
		main.mathpanel[5].repaint();

	}

	/**
	 * Zeichnet Einzellast
	 */
	void draw1(double x, double z) {
		/*
		 * first.setCoordinates(x, z); first.setRadius(4);
		 * main.mathpanel[5].add(first);
		 */
		high.setCoordinates(x, z);
		high.setRadius(4);
		main.mathpanel[5].add(high);
		main.one[0].setBase(new Punkt2D(x, z));
		main.one[0].setHead(new Punkt2D(x, 0));
		main.one[0].setVisible(true);
		high.setVisible(true);
		main.mathpanel[5].repaint();
	}

	/**
	 * Zeichnet Gleichstreckenlast
	 */
	void draw2(double x, double y, double z) {
		high.setCoordinates((x + y) / 2, z);
		high.setRadius(4);
		main.mathpanel[5].add(high);
		main.one[0].setBase(new Punkt2D(x, z));
		main.one[0].setHead(new Punkt2D(x, 0));
		main.one[0].setVisible(true);
		for (int i = 1; i < 10; i++) {
			if (i * main.length / 10 > x && i * main.length / 10 < y) {
				main.one[i].setBase(new Punkt2D(i * main.length / 10, z));
				main.one[i].setHead(new Punkt2D(i * main.length / 10, 0));
				main.one[i].setVisible(true);
			}
			if (i * main.length / 10 > y && i * main.length / 10 < x) {
				main.one[i].setBase(new Punkt2D(i * main.length / 10, z));
				main.one[i].setHead(new Punkt2D(i * main.length / 10, 0));
				main.one[i].setVisible(true);
			}
		}
		main.one[10].setBase(new Punkt2D(y, z));
		main.one[10].setHead(new Punkt2D(y, 0));
		main.one[10].setVisible(true);
		last.setCoordinates(y, 0);
		last.setRadius(4);
		main.mathpanel[5].add(last);
		main.line.setPoints(x, z, y, z);
		last.setVisible(true);
		high.setVisible(true);
		main.line.setVisible(true);
		main.mathpanel[5].repaint();
	}

	/**
	 * Zeichnet Dreiecksstreckenlast
	 */
	void draw3(double x, double y, double z) {
		for (int i = 1; i < 10; i++) {
			if (i * main.length / 10 > x && i * main.length / 10 < y) {
				main.one[i].setBase(new Punkt2D(main.length * i / 10, z
						/ (y - x) * (i * main.length / 10 - x)));
				main.one[i].setHead(new Punkt2D(main.length * i / 10, 0));
				main.one[i].setVisible(true);
			}
			if (i * main.length / 10 > y && i * main.length / 10 < x) {
				main.one[i].setBase(new Punkt2D(main.length * i / 10, z
						/ (y - x) * (i * main.length / 10 - x)));
				main.one[i].setHead(new Punkt2D(main.length * i / 10, 0));
				main.one[i].setVisible(true);
			}
		}
		main.one[10].setBase(new Punkt2D(y, z));
		main.one[10].setHead(new Punkt2D(y, 0));
		main.one[10].setVisible(true);
		high.setCoordinates(y, z);
		high.setRadius(4);
		main.mathpanel[5].add(high);
		main.line.setP1(new Punkt2D(x, 0));
		main.line.setP2(high.getPunkt2D());
		last.setVisible(false);
		high.setVisible(true);
		main.line.setVisible(true);
		main.mathpanel[5].repaint();
	}

	/**
	 * Zeichnet sinusfrmige streckenlast
	 */
	void draw4(double z) {
		high.setCoordinates(main.length / 2, z);
		high.setRadius(4);
		main.mathpanel[5].add(high);
		for (int i = 1; i < 10; i++) {
			main.one[i].setBase(new Punkt2D(i * main.length / 10, z
					* Math.sin(i * Math.PI / 10)));
			main.one[i].setHead(new Punkt2D(i * main.length / 10, 0));
			main.one[i].setVisible(true);
		}
		try {
			main.function = Function1DParser
					.parse("x", new String(Double.toString(z) + "*sin(x*"
							+ Double.toString(Math.PI / main.length) + ")"));
			main.func.setPoints(GraphUtil.compute(main.function, 0,
					main.length, 400));
		} catch (Exception e) {
		}
		main.func.setVisible(true);
		high.setVisible(true);
		main.mathpanel[5].repaint();
	}

	/**
	 * Zeichnet die Graphen in den Mathpanels
	 */
	void compute() {
		if (main.lager == 2) {
			switch (main.art) {
			case 1:
				drawgraph1();
				break;
			case 2:
				drawgraph2();
				break;

			case 3:
				drawgraph3();
				break;

			case 4:
				drawgraph4();
				break;
			default:
				break;
			}

		} else {
			switch (main.art) {
			case 1:
				drawgraphk1();
				break;
			case 2:
				drawgraphk2();
				break;

			case 3:
				drawgraphk3();
				break;

			case 4:
				drawgraphk4();
				break;
			default:
				break;
			}

		}
	}

	void drawgraph1() {
		Vector points = new Vector();
		double length = main.length / 100;
		double firstx = high.getX() / 100;
		double highy = high.getY() / 100;
		double b = firstx * highy / length;
		double a = highy - b;
		power[1].setHead(new Punkt2D(firstx, 0));
		power[1].setBase(new Punkt2D(firstx, highy));
		power[0].setBase(new Punkt2D(0, -a));
		power[2].setHead(new Punkt2D(length, 0));
		power[2].setBase(new Punkt2D(length, -b));
		main.mathpanel[0].getScene2d().setLimits(0, length, -1, 1);
		points.add(new Punkt2D(0, 0));
		points.add(new Punkt2D(0, a));
		points.add(new Punkt2D(firstx, a));
		points.add(new Punkt2D(firstx, -b));
		points.add(new Punkt2D(length, -b));
		points.add(new Punkt2D(length, 0));
		main.graph[1].setPoints(points);
		main.mathpanel[1].getScene2d().setLimits(0, length, -1, 1);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(highy * length * (length - firstx)
							/ (6 * (length * length)))
					+ "*6*x");
			points.addAll(GraphUtil.compute(function, 0, firstx, 100));
			function = Function1DParser.parse("x", Double.toString(-highy
					* length * (length - firstx))
					+ "*(-x*"
					+ Double.toString(1 / (length * length))
					+ "+(x-"
					+ Double.toString(firstx)
					+ ")*"
					+ Double.toString(1 / (length * (length - firstx))) + ")");
			points.addAll(GraphUtil.compute(function, firstx, length, 100));
		} catch (Exception e) {
		}
		main.graph[2].setPoints(points);
		main.mathpanel[2].getScene2d().setLimits(0, length, 0, 0.3);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy * length * (length - firstx) / 6)
					+ "*(1-(3*x^2+"
					+ Double.toString(Math.pow(length - firstx, 2))
					+ ")*"
					+ Double.toString(1 / (length * length)) + ")");
			points.addAll(GraphUtil.compute(function, 0, firstx, 100));
			function = Function1DParser.parse("x", Double.toString(-highy
					* length * (length - firstx) / 6)
					+ "*(1-(3*x^2+"
					+ Double.toString(Math.pow(length - firstx, 2))
					+ ")*"
					+ Double.toString(1 / (length * length))
					+ "+3*(x-"
					+ Double.toString(firstx)
					+ ")^2*"
					+ Double.toString(1 / (length * (length - firstx))) + ")");
			points.addAll(GraphUtil.compute(function, firstx, length, 100));
		} catch (Exception e) {
		}
		main.graph[3].setPoints(points);
		main.mathpanel[3].getScene2d().setLimits(0, length, -0.1, 0.1);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy * length * (length - firstx) / 6)
					+ "*(x-x*(x^2+"
					+ Double.toString(Math.pow(length - firstx, 2))
					+ ")*"
					+ Double.toString(1 / (length * length)) + ")");
			points.addAll(GraphUtil.compute(function, 0, firstx, 100));
			function = Function1DParser.parse("x", Double.toString(-highy
					* length * (length - firstx) / 6)
					+ "*(x-x*(x^2+"
					+ Double.toString(Math.pow(length - firstx, 2))
					+ ")*"
					+ Double.toString(1 / (length * length))
					+ "+(x-"
					+ Double.toString(firstx)
					+ ")^3*"
					+ Double.toString(1 / (length * (length - firstx))) + ")");
			points.addAll(GraphUtil.compute(function, firstx, length, 100));
		} catch (Exception e) {
		}
		main.graph[4].setPoints(points);
		main.mathpanel[4].getScene2d().setLimits(0, length, -0.06, 0);
		for (int i = 0; i < 5; i++)
			main.mathpanel[i].repaint();
		return;
	}

	void drawgraph2() {
		Vector points = new Vector();
		double length = main.length / 100;
		double lastx = last.getX() / 100;
		double highy = high.getY() / 100;
		double b = Math.pow(lastx, 2) / 4 * highy / length;
		double a = highy * (lastx) / 2 - b;
		points.add(new Punkt2D(0, 0));
		points.add(new Punkt2D(0, highy));
		points.add(new Punkt2D(lastx, highy));
		points.add(new Punkt2D(lastx, 0));
		points.add(new Punkt2D(length, 0));
		power[0].setBase(new Punkt2D(0, -a));
		power[2].setHead(new Punkt2D(length, 0));
		power[2].setBase(new Punkt2D(length, -b));
		main.graph[0].setPoints(points);
		main.mathpanel[0].getScene2d().setLimits(0, length, -1, 1);
		points = new Vector();
		points.add(new Punkt2D(0, 0));
		points.add(new Punkt2D(0, a));
		points.add(new Punkt2D(0, a));
		points.add(new Punkt2D(lastx, -b));
		points.add(new Punkt2D(length, -b));
		points.add(new Punkt2D(length, 0));
		main.graph[1].setPoints(points);
		main.mathpanel[1].getScene2d().setLimits(0, length, -0.3, 0.3);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy / 2)
					+ "*(x^2-(1-"
					+ Double.toString(Math.pow((length - lastx) / length, 2))
					+ ")*x*" + Double.toString(length) + ")");
			points.addAll(GraphUtil.compute(function, 0, lastx, 100));
			function = Function1DParser.parse("x", Double.toString(-highy / 2)
					+ "*(x^2-(x-" + Double.toString(lastx) + ")^2-(1-"
					+ Double.toString(Math.pow((length - lastx) / length, 2))
					+ ")*x*" + Double.toString(length) + ")");
			points.addAll(GraphUtil.compute(function, lastx, length, 100));
		} catch (Exception e) {
		}
		main.graph[2].setPoints(points);
		main.mathpanel[2].getScene2d().setLimits(0, length, 0, 0.3);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy / 24)
					+ "*(4*x^3-6*(1-"
					+ Double.toString(Math.pow((length - lastx) / length, 2))
					+ ")*x^2*"
					+ Double.toString(length)
					+ "+(1-"
					+ Double.toString(Math.pow((length - lastx) / length, 2))
					+ " )^2*" + Double.toString(Math.pow(length, 3)) + ")");
			points.addAll(GraphUtil.compute(function, 0, lastx, 100));
			function = Function1DParser.parse("x", Double.toString(-highy / 24)
					+ "*(4*x^3-4*(x-" + Double.toString(lastx) + ")^3-6*(1-"
					+ Double.toString(Math.pow((length - lastx) / length, 2))
					+ ")*x^2*" + Double.toString(length) + "+(1-"
					+ Double.toString(Math.pow((length - lastx) / length, 2))
					+ " )^2*" + Double.toString(Math.pow(length, 3)) + ")");
			points.addAll(GraphUtil.compute(function, lastx, length, 100));
		} catch (Exception e) {
		}
		main.graph[3].setPoints(points);
		main.mathpanel[3].getScene2d().setLimits(0, length, -0.1, 0.1);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy / 24)
					+ "*(x^4-2*(1-"
					+ Double.toString(Math.pow((length - lastx) / length, 2))
					+ ")*x^3*"
					+ Double.toString(length)
					+ "+(1-"
					+ Double.toString(Math.pow((length - lastx) / length, 2))
					+ " )^2*x*" + Double.toString(Math.pow(length, 3)) + ")");
			points.addAll(GraphUtil.compute(function, 0, lastx, 100));
			function = Function1DParser.parse("x", Double.toString(-highy / 24)
					+ "*(x^4-(x-" + Double.toString(lastx) + ")^4-2*(1-"
					+ Double.toString(Math.pow((length - lastx) / length, 2))
					+ ")*x^3*" + Double.toString(length) + "+(1-"
					+ Double.toString(Math.pow((length - lastx) / length, 2))
					+ " )^2*x*" + Double.toString(Math.pow(length, 3)) + ")");
			points.addAll(GraphUtil.compute(function, lastx, length, 100));
		} catch (Exception e) {
		}
		main.graph[4].setPoints(points);
		main.mathpanel[4].getScene2d().setLimits(0, length, -0.05, 0);
		for (int i = 0; i < 5; i++)
			main.mathpanel[i].repaint();
		return;
	}

	void drawgraph3() {
		Vector points = new Vector();
		double length = main.length / 100;
		double lastx = length;
		double highy = high.getY() / 100;
		double b = 2 * lastx * lastx * highy / (6 * length);
		double a = highy * lastx / 2 - b;
		points.add(new Punkt2D(0, 0));
		points.add(new Punkt2D(0, 0));
		points.add(new Punkt2D(lastx, highy));
		points.add(new Punkt2D(lastx, 0));
		points.add(new Punkt2D(length, 0));
		power[0].setBase(new Punkt2D(0, -a));
		power[2].setHead(new Punkt2D(length, 0));
		power[2].setBase(new Punkt2D(length, -b));
		main.graph[0].setPoints(points);
		main.mathpanel[0].getScene2d().setLimits(0, length, -1, 1);
		points = new Vector();
		points.add(new Punkt2D(0, 0));
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy / (60 * length))
					+ "*(" + Double.toString(-Math.pow(length, 2)) + "+3*x^2)");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		points.add(new Punkt2D(length, 0));
		main.graph[1].setPoints(points);
		main.mathpanel[1].getScene2d().setLimits(0, length, -0.03, 0.03);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy / (60 * length))
					+ "*(-x*" + Double.toString(Math.pow(length, 2)) + "+x^3)");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		main.graph[2].setPoints(points);
		main.mathpanel[2].getScene2d().setLimits(0, length, 0, 0.015);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy / (360 * length))
					+ "*(7*"
					+ Double.toString(Math.pow(length, 4))
					+ "-30*x^2*"
					+ Double.toString(Math.pow(length, 2))
					+ "+15*x^4)");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		main.graph[3].setPoints(points);
		main.mathpanel[3].getScene2d().setLimits(0, length, -0.04, 0.04);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy / (360 * length))
					+ "*(7*x*"
					+ Double.toString(Math.pow(length, 4))
					+ "-10*x^3*"
					+ Double.toString(Math.pow(length, 2))
					+ "+3*x^5)");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		main.graph[4].setPoints(points);
		main.mathpanel[4].getScene2d().setLimits(0, length, -0.03, 0);
		for (int i = 0; i < 5; i++)
			main.mathpanel[i].repaint();
		return;
	}

	void drawgraph4() {
		Vector points = new Vector();
		double length = main.length / 100;
		double highy = high.getY() / 100;
		double b = highy * length / (Math.PI);
		double a = b;
		try {
			main.function = Function1DParser.parse("x", new String(Double
					.toString(highy)
					+ "*sin(x*" + Double.toString(Math.PI / length) + ")"));
			points.addAll(GraphUtil.compute(main.function, 0, length, 100));
		} catch (Exception e) {
		}
		power[0].setBase(new Punkt2D(0, -a));
		power[2].setHead(new Punkt2D(length, 0));
		power[2].setBase(new Punkt2D(length, -b));
		main.graph[0].setPoints(points);
		main.mathpanel[0].getScene2d().setLimits(0, length, -1, 1);
		points = new Vector();
		points.add(new Punkt2D(0, 0));
		points.add(new Punkt2D(0, a));
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(highy * length / Math.PI)
					+ "*(cos(x*" + Double.toString(Math.PI / length) + ")");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		points.add(new Punkt2D(length, -b));
		points.add(new Punkt2D(length, 0));
		main.graph[1].setPoints(points);
		main.mathpanel[1].getScene2d().setLimits(0, length, -0.4, 0.4);
		points = new Vector();
		points.add(new Punkt2D(0, 0));
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(highy * Math.pow(length / Math.PI, 2))
					+ "*(sin(x*" + Double.toString(Math.PI / length) + "))");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		points.add(new Punkt2D(length, 0));
		main.graph[2].setPoints(points);
		main.mathpanel[2].getScene2d().setLimits(0, length,0, 0.2);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-Math.pow(length / Math.PI, 3) * highy)
					+ "*(cos(x*" + Double.toString(Math.PI / length) + "))");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		main.graph[3].setPoints(points);
		main.mathpanel[3].getScene2d().setLimits(0, length, -0.1, 0.1);
		points = new Vector();
		points.add(new Punkt2D(0, 0));
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy * Math.pow(length / Math.PI, 4))
					+ "*(sin(x*" + Double.toString(Math.PI / length) + "))");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		points.add(new Punkt2D(length, 0));
		main.graph[4].setPoints(points);
		main.mathpanel[4].getScene2d().setLimits(0, length, -0.05, 0);
		for (int i = 0; i < 5; i++)
			main.mathpanel[i].repaint();
		return;
	}

	void drawgraphk1() {
		Vector points = new Vector();
		double length = main.length / 100;
		double firstx = high.getX() / 100;
		double highy = high.getY() / 100;
		double a = highy;
		power[1].setHead(new Punkt2D(firstx, 0));
		power[1].setBase(new Punkt2D(firstx, highy));
		power[0].setBase(new Punkt2D(0, -a));
		main.mathpanel[0].getScene2d().setLimits(0, length, -1, 1);
		points.add(new Punkt2D(0, 0));
		points.add(new Punkt2D(0, a));
		points.add(new Punkt2D(firstx, a));
		points.add(new Punkt2D(firstx, 0));
		points.add(new Punkt2D(length, 0));
		main.graph[1].setPoints(points);
		main.mathpanel[1].getScene2d().setLimits(0, length, -1, 1);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(highy)
					+ "*(" + Double.toString(firstx) + "-x)");
			points.addAll(GraphUtil.compute(function, 0, firstx, 100));
		} catch (Exception e) {
		}
		points.add(new Punkt2D(length, 0));
		main.graph[2].setPoints(points);
		main.mathpanel[2].getScene2d().setLimits(0, length, 0, 1.5);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy / 2)
					+ "*(2*x*" + Double.toString(firstx) + "-x^2)");
			points.addAll(GraphUtil.compute(function, 0, firstx, 100));
			function = Function1DParser.parse("x", Double.toString(-highy / 2)
					+ "*(2*x*" + Double.toString(firstx) + "-x^2+(x-"
					+ Double.toString(firstx) + ")^2)");
			points.addAll(GraphUtil.compute(function, firstx, length, 100));
		} catch (Exception e) {
		}
		main.graph[3].setPoints(points);
		main.mathpanel[3].getScene2d().setLimits(0, length, -1, 0);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy / 6)
					+ "*(3*x^2*" + Double.toString(firstx) + "-x^3)");
			points.addAll(GraphUtil.compute(function, 0, firstx, 100));
			function = Function1DParser.parse("x", Double.toString(-highy / 6)
					+ "*(3*x^2*" + Double.toString(firstx) + "-x^3+(x-"
					+ Double.toString(firstx) + ")^3)");
			points.addAll(GraphUtil.compute(function, firstx, length, 100));
		} catch (Exception e) {
		}
		main.graph[4].setPoints(points);
		main.mathpanel[4].getScene2d().setLimits(0, length, -1.2, 0);
		for (int i = 0; i < 5; i++)
			main.mathpanel[i].repaint();
		return;
	}

	void drawgraphk2() {
		Vector points = new Vector();
		double length = main.length / 100;
		double lastx = last.getX() / 100;
		double highy = high.getY() / 100;
		double a = highy * (length - lastx);
		points.add(new Punkt2D(0, 0));
		points.add(new Punkt2D(lastx, 0));
		points.add(new Punkt2D(lastx, highy));
		points.add(new Punkt2D(length, highy));
		points.add(new Punkt2D(length, 0));
		power[0].setBase(new Punkt2D(0, -a));
		main.graph[0].setPoints(points);
		main.mathpanel[0].getScene2d().setLimits(0, length, -1, 1);
		points = new Vector();
		points.add(new Punkt2D(0, 0));
		points.add(new Punkt2D(0, a));
		points.add(new Punkt2D(lastx, a));
		points.add(new Punkt2D(length, 0));
		main.graph[1].setPoints(points);
		main.mathpanel[1].getScene2d().setLimits(0, length, 0, 1);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(highy / 12 * (length - lastx))
					+ "*(-12*x+6*" + Double.toString(length + lastx) + ")");
			points.addAll(GraphUtil.compute(function, 0, lastx, 100));
			function = Function1DParser.parse("x", Double.toString(highy / 24
					* (length - lastx))
					+ "*(-24*x+12*"
					+ Double.toString(length + lastx)
					+ "+12*(x-"
					+ Double.toString(lastx)
					+ ")^2*"
					+ Double.toString(1 / (length - lastx)) + ")");
			points.addAll(GraphUtil.compute(function, lastx, length, 100));
		} catch (Exception e) {
		}
		main.graph[2].setPoints(points);
		main.mathpanel[2].getScene2d().setLimits(0, length,0, 1);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy / 12 * (length - lastx))
					+ "*(-6*x^2+6*" + Double.toString(length + lastx) + "*x)");
			points.addAll(GraphUtil.compute(function, 0, lastx, 100));
			function = Function1DParser.parse("x", Double.toString(-highy / 24
					* (length - lastx))
					+ "*(-12*x^2+12*"
					+ Double.toString(length + lastx)
					+ "*x+4*(x-"
					+ Double.toString(lastx)
					+ ")^3*"
					+ Double.toString(1 / (length - lastx)) + ")");
			points.addAll(GraphUtil.compute(function, lastx, length, 100));
		} catch (Exception e) {
		}
		main.graph[3].setPoints(points);
		main.mathpanel[3].getScene2d().setLimits(0, length, -0.6, 0);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser
					.parse("x", Double.toString(-highy / 12 * (length - lastx))
							+ "*(-2*x^3+3*" + Double.toString(length + lastx)
							+ "*x^2)");
			points.addAll(GraphUtil.compute(function, 0, lastx, 100));
			function = Function1DParser.parse("x", Double.toString(-highy / 24
					* (length - lastx))
					+ "*(-4*x^3+6*"
					+ Double.toString(length + lastx)
					+ "*x^2+(x-"
					+ Double.toString(lastx)
					+ ")^4*"
					+ Double.toString(1 / (length - lastx)) + ")");
			points.addAll(GraphUtil.compute(function, lastx, length, 100));
		} catch (Exception e) {
		}
		main.graph[4].setPoints(points);
		main.mathpanel[4].getScene2d().setLimits(0, length, -0.8, 0);
		for (int i = 0; i < 5; i++)
			main.mathpanel[i].repaint();
		return;
	}

	void drawgraphk3() {
		Vector points = new Vector();
		double length = main.length / 100;
		double highy = high.getY() / 100;
		double a = 2 * length * length * highy / 3;
		points.add(new Punkt2D(0, 0));
		points.add(new Punkt2D(0, highy));
		points.add(new Punkt2D(length, 0));
		power[0].setBase(new Punkt2D(0, -a));
		main.graph[0].setPoints(points);
		main.mathpanel[0].getScene2d().setLimits(0, length, -1, 1);
		points = new Vector();
		points.add(new Punkt2D(0, 0));
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy / (120 * length))
					+ "*(-60*"
					+ Double.toString(length)
					+ "+120*x-60*x^2*"
					+ Double.toString(1 / length) + ")");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		points.add(new Punkt2D(length, 0));
		main.graph[1].setPoints(points);
		main.mathpanel[1].getScene2d().setLimits(0, length, 0, 0.5);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(highy / (120 * length))
					+ "*(20*"
					+ Double.toString(Math.pow(length, 2))
					+ "-60*x*"
					+ Double.toString(length)
					+ "+60*x^2-20*x^3*"
					+ Double.toString(1 / length) + ")");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		main.graph[2].setPoints(points);
		main.mathpanel[2].getScene2d().setLimits(0, length, 0, 0.2);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy / (120 * length))
					+ "*(20*x*"
					+ Double.toString(Math.pow(length, 2))
					+ "-30*x^2*"
					+ Double.toString(length)
					+ "+20*x^3-5*x^4*"
					+ Double.toString(1 / length) + ")");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		main.graph[3].setPoints(points);
		main.mathpanel[3].getScene2d().setLimits(0, length, -0.1, 0);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-highy / (120 * length))
					+ "*(10*x^2*"
					+ Double.toString(Math.pow(length, 2))
					+ "-10*x^3*"
					+ Double.toString(length)
					+ "+5*x^4-x^5*"
					+ Double.toString(1 / length) + ")");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		main.graph[4].setPoints(points);
		main.mathpanel[4].getScene2d().setLimits(0, length, -0.1, 0);
		for (int i = 0; i < 5; i++)
			main.mathpanel[i].repaint();
		return;
	}

	void drawgraphk4() {
		Vector points = new Vector();
		double length = main.length / 100;
		double w = length / Math.PI;
		double highy = high.getY() / 100;
		double a = 2*highy * length / (Math.PI);
		try {
			main.function = Function1DParser.parse("x", new String(Double
					.toString(highy)
					+ "*sin(x*" + Double.toString(Math.PI / length) + ")"));
			points.addAll(GraphUtil.compute(main.function, 0, length, 100));
		} catch (Exception e) {
		}
		power[0].setBase(new Punkt2D(0, -a));
		main.graph[0].setPoints(points);
		main.mathpanel[0].getScene2d().setLimits(0, length, -1, 1);
		points = new Vector();
		points.add(new Punkt2D(0, 0));
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(highy*w)
					+ "*(cos(x*"
					+ Double.toString(Math.PI / length)
					+ ")"+"+1)");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		main.graph[1].setPoints(points);
		main.mathpanel[1].getScene2d().setLimits(0, length, 0, 0.9);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(w*highy)
					+ "*(-sin(x*"
					+ Double.toString(Math.PI / length)
					+ ")*"
					+ Double.toString(w)
					+ "-x+"
					+ Double.toString(Math.PI*w) +")");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		main.graph[2].setPoints(points);
		main.mathpanel[2].getScene2d().setLimits(0, length, 0, 0.7);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-w*highy)
					+ "*(cos(x*"
					+ Double.toString(Math.PI / length)
					+ ")*"
					+ Double.toString(Math.pow(w,2))
					+ "-1/2*x^2+x*"
					+ Double.toString(Math.PI*w)
					+ "-"
					+ Double.toString(Math.pow(w, 2)) + ")");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		main.graph[3].setPoints(points);
		main.mathpanel[3].getScene2d().setLimits(0, length, -0.3, 0);
		points = new Vector();
		try {
			ParsedFunction1D function = Function1DParser.parse("x", Double
					.toString(-w*highy)
					+ "*(sin(x*"
					+ Double.toString(Math.PI / length)
					+ ")*"
					+ Double.toString(Math.pow(w, 3))
					+ "-x^3/6+x^2/2*"
					+ Double.toString(Math.PI*w)
					+ "-x*"
					+ Double.toString(Math.pow(w, 2)) + ")");
			points.addAll(GraphUtil.compute(function, 0, length, 100));
		} catch (Exception e) {
		}
		main.graph[4].setPoints(points);
		main.mathpanel[4].getScene2d().setLimits(0, length, -0.4, 0);
		for (int i = 0; i < 5; i++)
			main.mathpanel[i].repaint();
		return;
	}
}