package uibk.mtk.draw2d.axes2d;

import java.awt.Graphics2D;
import java.awt.Point;
import java.util.Vector;

import uibk.mtk.draw2d.base.Scene2D;
import uibk.mtk.geom.geom2d.Punkt2D;

/**
 * Superklasse f?r die Achsen
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */

public abstract class Axis {

	/** Das 2D-Achsenobjekt zu der die Achse geh?rt */
	Axes2D axes;

	/** Die Ticks dieser Achse */
	Vector ticks;

	/** Beschriftung der Achse */
	String caption = ""; //$NON-NLS-1$

	/** Stil der Achse Axes2D.AXES_STYLE_TICKS,... */
	int style;

	/** Abstand Beschrifung - Tick */
	static final int LABELGAP = 4;

	/** L?nge des Pfeils am Ende der Achse */
	static int ARROWLENGTH = 18;

	/** Offset der "Fl?gel" des Pfeils am Ende der Achse */
	static int OFFSETWINGS = 5;

	/** L?nge der Pfeilspitze */
	static int LENGTHARROWHEAD = 5;

	/** Startpunkt der Achse in Ger?tekoordinaten */
	Point startPix = new Point();

	/** Endpunkt der Achse in Ger?tekoordinaten */
	Point endPix = new Point();

	/** Position des Names der Achse in Ger?tekoordinaten */
	Point captionpos = new Point();

	/** Pfeil zeigt nach rechts */
	static final int ARROW_RIGHT = 0;

	/** Pfeil zeigt nach links */
	static final int ARROW_LEFT = 1;

	/** Pfeil zeigt nach oben */
	static final int ARROW_UP = 2;

	/** Pfeil zeigt nach unten */
	static final int ARROW_DOWN = 3;

	/** setzt die Ticks f?r diese Achse */
	void setTicks(Vector ticks) {
		this.ticks = ticks;
	}

	/** Zeichnen der Achse */
	abstract void draw(Graphics2D g2, Scene2D coords2d);

	/** Erzeugen der Achse */
	public Axis(Axes2D axes) {
		this.axes = axes;

	}

	/**
	 * Zeichnet den Pfeil am Ende der Achse
	 * 
	 * @param g2
	 *            Graphikkontext
	 * @param x
	 *            x-Koordinate des Endpunktes der Achse
	 * @param y
	 *            y-Koordinate des Endpunktes der Achse
	 * @param direction
	 *            Richtung des Pfeils (ARROW_RIGHT,...)
	 */
	void drawArrow(Graphics2D g2, int x, int y, int direction) {
		switch (direction) {
		case (ARROW_UP):
			g2.drawLine(x, y, x, y - ARROWLENGTH);
			g2.drawLine(x, y - ARROWLENGTH, x - OFFSETWINGS, y - ARROWLENGTH
					+ LENGTHARROWHEAD);
			g2.drawLine(x, y - ARROWLENGTH, x + OFFSETWINGS, y - ARROWLENGTH
					+ LENGTHARROWHEAD);
			break;
		case (ARROW_DOWN):
			g2.drawLine(x, y, x, y + ARROWLENGTH);
			g2.drawLine(x, y + ARROWLENGTH, x - OFFSETWINGS, y + ARROWLENGTH
					- LENGTHARROWHEAD);
			g2.drawLine(x, y + ARROWLENGTH, x + OFFSETWINGS, y + ARROWLENGTH
					- LENGTHARROWHEAD);
			break;
		case (ARROW_RIGHT):
			g2.drawLine(x, y, x + ARROWLENGTH, y);
			g2.drawLine(x + ARROWLENGTH - LENGTHARROWHEAD, y + OFFSETWINGS, x
					+ ARROWLENGTH, y);
			g2.drawLine(x + ARROWLENGTH - LENGTHARROWHEAD, y - OFFSETWINGS, x
					+ ARROWLENGTH, y);
			break;
		case (ARROW_LEFT):
			g2.drawLine(x, y, x - ARROWLENGTH, y);
			g2.drawLine(x - ARROWLENGTH + LENGTHARROWHEAD, y + OFFSETWINGS, x
					- ARROWLENGTH, y);
			g2.drawLine(x - ARROWLENGTH + LENGTHARROWHEAD, y - OFFSETWINGS, x
					- ARROWLENGTH, y);
			break;
		default:
			break;

		}
	}

	abstract void compute(Punkt2D start, Punkt2D end, Punkt2D axisIntersection,
			int tickAndLabelPos, Graphics2D g2, Scene2D scene2d);

	void setStyle(int style) {
		this.style = style;
	}

}