package uibk.mtk.draw2d.base;

import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.image.BufferedImage;

import uibk.mtk.lang.Highlightable;
import uibk.mtk.lang.MathPanel;
import uibk.mtk.lang.PrepaintComputable;

/**
 * Basisklasse f?r alle Aktionen der 2D-Applets.
 * 
 * <p>
 * Der Panel enth?lt eine <CODE>Scene2D</CODE> in der die zu diesem Panel
 * hinzugef?gten <CODE>Drawable2D</CODE> Objekte "leben". Der Mathpanel2D
 * verwaltet das Zeichnen dieser Objekte, erlaubt Verschieben mit der Maus von
 * Dragable Objekten und zeigt gegebenfalls Tooltips der Objekte an (siehe
 * Interface <CODE>Tooltipable</CODE>) oder hebt Sie hervor (siehe Interface
 * <CODE>Highlightable</CODE>).
 * </p>
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class MathPanel2D extends MathPanel {

	/** die 2D-Szene, die der Panel enth?lt */
	private Scene2D scene2d;
	
	int tmp;

	/** Erzeugt den Mathpanel mit einer 2D-Szene in den Koordinaten (-1,1,-1,1) */
	public MathPanel2D() {

		scene2d = new Scene2D(this);

	}

	/** gibt die 2D-Szene zur?ck */
	public Scene2D getScene2d() {
		return scene2d;

	}

	/**
	 * gibt zur?ck ob sich ein Punkt (z.B. vom einem Mouse-Event) im
	 * Koordinatensystem befindet (oder au?erhalb, d.h. am Rand)
	 */
	public boolean isPointInCoordsArea(Point point) {
		Insets insets = scene2d.getInsets();
		int edge = scene2d.getEdge();

		if (point.x > insets.left + edge
				&& point.x < getWidth() - insets.right - edge
				&& point.y < getHeight() - insets.bottom - edge
				&& point.y > insets.top + edge) {
			return true;
		}

		return false;
	}

	/**
	 * Hinzuf?gen eines <CODE>Drawable</CODE> Objektes zum MathPanel. Falls
	 * <CODE>obj</CODE> nicht vom Typ <CODE>Drawable2D</CODE> oder <CODE>
	 * null</CODE> ist wird ein <CODE>IllegalArgumentException</CODE>
	 * geworfen. Falls es schon hinzugef?gt worden ist, passiert nichts.
	 */
	public void add(Object obj) {
		if (obj == null || !(obj instanceof Drawable2D))
			throw new IllegalArgumentException(""); //$NON-NLS-1$

		if (!drawItems.contains(obj)) {
			Drawable2D draw = (Drawable2D) obj;

			super.add(draw);
			draw.setMathPanel2D(this);
		}
	}

	/** Zeichnen der Drawable-Objekte */
	public void drawDrawables(BufferedImage framebuffer, Graphics2D g2) {
		
		if(options!=null)
		   options.setRenderingHints(g2);
		
		int ct = drawItems.size();

		scene2d.prepareScene();

		if (repaintrequested) {
			scene2d.resetInsets();

			for (int i = 0; i < ct; i++) {
				if (drawItems.elementAt(i) instanceof PrepaintComputable
						&& ((Drawable2D) drawItems.elementAt(i)).isVisible()) {
					try {
						((PrepaintComputable) drawItems.elementAt(i))
								.prepaintcompute();
					} catch (Exception ex) {

						reportError(ex);
					}
				}
			}

			//m?gliche Request durchf?hren
			boolean changes = scene2d.processRequests();

			if (changes) { //neu berechnen

				for (int i = 0; i < ct; i++) {
					if (drawItems.elementAt(i) instanceof PrepaintComputable
							&& ((Drawable2D) drawItems.elementAt(i))
									.isVisible()) {
						try {
							((PrepaintComputable) drawItems.elementAt(i))
									.prepaintcompute();
						} catch (Exception ex) {

						}
					}
				}

			}
		}

		//Zeichnen der Objekte
		for (int i = 0; i < ct; i++) {
			Drawable2D d = (Drawable2D) drawItems.elementAt(i);
			if (d.isVisible()) {

				if (d == highlighted) {

					((Highlightable) d).highlight(framebuffer, g2);
				} else
					d.draw(framebuffer, g2);

			}
		}

		this.repaintrequested = false;

	}

}