package uibk.mtk.draw2d.objects;

import java.awt.event.MouseEvent;

import uibk.mtk.lang.Dragable;

/**
 * Ein 2 dimensionaler Punkt in der Ebene, der mit der Maus verschoben werden
 * kann (und alle Eigenschaften eines MathPoint2D hat)
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 *  
 */
public class DragablePoint2D extends MathPoint2D implements Dragable {

	/** ist Dragging aktiviert */
	boolean dragenabled = true;

	/** Gibt zur?ck ob das Drag & Drop aktiviert ist */
	public boolean isDragEnabled() {
		return dragenabled;
	}

	/**
	 * Radius in Pixeln mit dem entschieden wird ob bei Punkt beim Dragging
	 * (Ziehen mit Maus) angesprochen ist, oder nicht (kann gr??er sein wie der
	 * Radius zum Zeichnen ->leichter zum Ziehen
	 */
	int containingradius = 5;

	/** Erzeugt einen verschiebbaren Punkt mit den Koordinaten (0,0) */
	public DragablePoint2D() {
		super();
	}

	/** Erzeugt einen verschiebbaren Punkt mit den angegebenen Koordinaten */
	public DragablePoint2D(double x, double y) {
		super(x, y);
	}

	/** Ziehen des Punktes, setzen der neuen Position */
	public void drag(MouseEvent ev) {
		if (!dragenabled)
			return;
		this.x = scene2d.pixelToX(ev.getX());
		this.y = scene2d.pixelToY(ev.getY());
	}

	/** Ziehen aktivieren/deaktivieren */
	public void enabledrag(boolean enable) {
		dragenabled = enable;
	}

	/**
	 * Setzt den Radius mit dem entschieden wird ob ob ein Punkt im Kreis
	 * enthalten ist, ist gedacht falls der Punkt draggable ist, damit er
	 * leicher zum "Ziehen" ist
	 */
	public void setContainingRadius(int r) {
		this.containingradius = r;
	}

	/**
	 * Gibt zur?ck, ob die Maus im "Ziehbereich" des Punktes ist (dieser wird
	 * durch die Kreis mit dem Radius containingradius festgesetzt)
	 *  
	 */
	public boolean dragcontains(int xm, int ym) {
		int r = radius < containingradius ? containingradius : radius;
		int xp = super.scene2d.xToPixel(x);
		int yp = super.scene2d.yToPixel(y);
		if ((xm - xp) * (xm - xp) + (ym - yp) * (ym - yp) > r * r) {
			return false;
		}
		return true;
	}
}