package uibk.mtk.draw2d.objects;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.image.BufferedImage;

import uibk.mtk.draw2d.base.Drawable2D;
import uibk.mtk.geom.geom2d.Punkt2D;

/**
 * Ein Linie mit ver?nderbarer Farbe und ver?nderbaren Linienstil in
 * Weltkoordinaten (Koordinaten in einer <CODE>Scene2D</CODE>).
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 *  
 */
public class MathLine2D extends Drawable2D {

	/** Stil zum Zeichnen der Linie */
	Stroke stroke;

	/** Anfangspunkt der Linie */
	Punkt2D a = new Punkt2D();

	/** Endpunkt der Linie */
	Punkt2D b = new Punkt2D();

	/**
	 * setzt Anfangspunkt und Endpunkt der Linie mit den Koordinaten der
	 * ?bergebenen Punkte
	 */
	public void setPoints(MathPoint2D p1, MathPoint2D p2) {
		a.x = p1.x;
		a.y = p1.y;
		b.x = p2.x;
		b.y = p2.y;
	}

	/**
	 * Erzeugt eine Linie der Linie mit dem angegeben Anfangs- und Endpunkt
	 * 
	 * @param x1
	 *            x-Koordinate 1.Punkt
	 * @param y1
	 *            y-Koordinate 1.Punkt
	 * @param x2
	 *            x-Koordinate 2.Punkt
	 * @param y2
	 *            y-Koordinate 2.Punkt
	 */
	public MathLine2D(double x1, double y1, double x2, double y2) {
		a.x = x1;
		a.y = y1;
		b.x = x2;
		b.y = y2;

	}

	/**
	 * setzt Anfangspunkt und Endpunkt der Linie mit den gebenen Koordinaten
	 * 
	 * @param x1
	 *            x-Koordinate 1.Punkt
	 * @param y1
	 *            y-Koordinate 1.Punkt
	 * @param x2
	 *            x-Koordinate 2.Punkt
	 * @param y2
	 *            y-Koordinate 2.Punkt
	 */
	public void setPoints(double x1, double y1, double x2, double y2) {
		a.x = x1;
		a.y = y1;
		b.x = x2;
		b.y = y2;
	}

	/** gibt die Koordinaten des 1.Punktes zur?ck */
	public Punkt2D getP1() {
		return new Punkt2D(a);
	}

	/** gibt die Koordinaten des 2.Punktes zur?ck */
	public Punkt2D getP2() {
		return new Punkt2D(b);
	}

	/** erzeugt eine (leere) Linie von (0,0) nach (0,0) */
	public MathLine2D() {
		stroke = new BasicStroke();
	}

	/** setzt den Linienstil */
	public void setStroke(Stroke stroke) {
		this.stroke = stroke;
	}

	/** setzt die Koordinaten des 1.Punktes */
	public void setP1(Punkt2D p) {
		a.setLocation(p);
	}

	/** setzt die Koordinaten des 2.Punktes */
	public void setP2(Punkt2D p) {
		b.setLocation(p);

	}

	/** Zeichnen der Linie */
	public void draw(BufferedImage framebuffer, Graphics2D g2) {
		g2.setColor(color);
		Stroke old = g2.getStroke();

		g2.setStroke(stroke);
		Point p = scene2d.project(a);
		Point q = scene2d.project(b);
		g2.drawLine(p.x, p.y, q.x, q.y);
		g2.setStroke(old);
	}

}