package uibk.mtk.draw2d.objects;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import uibk.mtk.draw2d.base.Drawable2D;
import uibk.mtk.geom.geom2d.Punkt2D;
import Jama.util.Maths;

/**
 * Eine 2 dimensionale Feder mit vernderbarer Breite,Dicke,Anzahl der Zacken
 * und Farbe.
 * <p>
 * Die Koordinaten sind Weltkoordinaten in einer <CODE>Scene2D</CODE>.
 * 
 * <p>
 * 
 * <p>
 * Copyright: Institut fr Technische Mathematik, Geometrie und Bauinformatik,
 * 2006
 * </p>
 * 
 * @author Andreas Rudigier (csae7164@uibk.ac.at)
 * 
 */
public class PhysicFeder2D extends Drawable2D {
	/** x-Koordinate der Anfangs- und Endpunkte im aktuellen Koordinatensystem */
	protected double x1, x2;

	/** x-Koordinate der Anfangs- und Endpunkte im aktuellen Koordinatensystem */
	protected double y1, y2;

	/** Breite der Feder (in Pixeln) */
	protected int sizeX;

	/** Dicke der Feder (in Pixeln) */
	protected int sizeY;

	/** Anzahl der Zacken */
	protected int sizeZ;

	/** Default Konstruktor* */
	public PhysicFeder2D() {
	}

	/**
	 * Konstruktor der eine Feder mit angegebnen Koordinaten anlegt
	 * 
	 * @param x
	 *            x-Koordinate
	 * @param y
	 *            y-Koordinate
	 */
	public PhysicFeder2D(double x1, double y1, double x2, double y2) {

		this.x1 = x1;
		this.y1 = y1;
		this.x2 = x2;
		this.y2 = y2;

	}

	/**
	 * Gibt x1-Koordinate zurck
	 * 
	 * @return x1 x1-Koordinate
	 */
	public double getX1() {
		return x1;

	}

	/**
	 * Gibt x2-Koordinate zurck
	 * 
	 * @return x2 x2-Koordinate
	 */

	public double getX2() {
		return x2;

	}

	/**
	 * Gibt y1-Koordinate zurck
	 * 
	 * @return y1 y1-Koordinate
	 */
	public double getY1() {
		return y1;

	}

	/**
	 * Gibt y2-Koordinate zurck
	 * 
	 * @return y2 y2-Koordinate
	 */
	public double getY2() {
		return y2;

	}

	/** gibt die x-Gre aus in Pixel */
	public int getSizeX() {
		return sizeX;
	}

	/** gibt die y-Gre aus in Pixel */
	public int getSizeY() {
		return sizeY;
	}

	/** gibt die z-Gre aus */
	public int getSizeZ() {
		return sizeZ;
	}

	/**
	 * Setzt die x1-Koordinate der Feder
	 * 
	 * @param x1
	 *            x1-Koordinate
	 */
	public void setX1(double x1) {
		this.x1 = x1;

	}

	/**
	 * Setzt die x2-Koordinate der Feder
	 * 
	 * @param x1
	 *            x2-Koordinate
	 */
	public void setX2(double x2) {
		this.x2 = x2;

	}

	/**
	 * Setzt die y1-Koordinate der Masse
	 * 
	 * @param y1
	 *            y1-Koordinate
	 */
	public void setY1(double y1) {
		this.y1 = y1;
	}

	/**
	 * Setzt die y2-Koordinate der Masse
	 * 
	 * @param y2
	 *            y2-Koordinate
	 */
	public void setY2(double y2) {
		this.y2 = y2;
	}

	/**
	 * setzt die Koordinaten der Feder
	 * 
	 * @param x
	 *            x-Koordinate
	 * @param y
	 *            y-Koordinate
	 */
	public void setCoordinates(double x1, double y1, double x2, double y2) {
		this.x1 = x1;
		this.y1 = y1;
		this.x2 = x2;
		this.y2 = y2;
	}

	/**
	 * Setzt die Gre der Feder zum Zeichnen fest
	 * 
	 * @param sizeX
	 *            x-Koordinate
	 * @param sizeY
	 *            y-Koordinate
	 */
	public void setSize(int sizeX, int sizeY, int sizeZ) {
		this.sizeX = sizeX;
		this.sizeY = sizeY;
		this.sizeZ = sizeZ;
	}

	/**
	 * Setzt die Koordinaten des Anfang- und Endpunktes auf die Koordinaten des
	 * Arguments
	 */
	public void setCoordinates(Punkt2D p, Punkt2D q) {
		this.x1 = p.x;
		this.y1 = p.y;
		this.x2 = q.x;
		this.y2 = q.y;

	}

	/** gibt die Koordinaten des AnfangPunktes zurck */
	public Punkt2D getAPunkt2D() {
		return new Punkt2D(x1, y1);
	}

	/** gibt die Koordinaten des EndPunktes zurck */
	public Punkt2D getEPunkt2D() {
		return new Punkt2D(x2, y2);
	}

	/**
	 * Zeichnen der Feder von Anfangs- bis Endpunkt und Breite und Dicke <CODE>sizeX,
	 * sizeY</CODE> in gesetzter Farbe und vorgegebener Anzahl der Zacken
	 */
	public void draw(BufferedImage framebuffer, Graphics2D g2) {

	/*	int xp1 = super.scene2d.xToPixel(x1);
		int yp1 = super.scene2d.yToPixel(y1);
		int xp2 = super.scene2d.xToPixel(x2);
		int yp2 = super.scene2d.yToPixel(y2);*/
		
		double yPixelScale = super.scene2d.getYPixelScale();

		double breite = sizeX*yPixelScale;
		int dicke = sizeY;
		double xli, yli, xs, ys, vor1, vor2;

		g2.setColor(color);

		// Setzen der Dicke der Feder
		g2.setStroke(new BasicStroke((dicke >= 0) ? dicke : -dicke));
		
		if (!(x1 == x2 & y1 == y2)) {
			// Zeichnen der Feder
			xli = ((x2 - x1) / (2 * sizeZ + 1));
			yli = ((y2 - y1) / (2 * sizeZ + 1));
			xs = (breite * (-yli) / Maths.hypot((xli), (yli)));
			ys = (breite * (xli) / Maths.hypot((xli), (yli)));
			
			int xp1 = super.scene2d.xToPixel(x1);
			int yp1 = super.scene2d.yToPixel(y1);
			int xp2 = super.scene2d.xToPixel(x1 + xs + xli);
			int yp2 = super.scene2d.yToPixel(y1 + ys + yli);			

			g2.drawLine(xp1, yp1, xp2, yp2);

			for (int i = 1; i < 2 * sizeZ; i++) {

				vor1 = Math.pow((-1), i + 1);
				vor2 = Math.pow((-1), i);

				xp1 = super.scene2d.xToPixel(x1 + vor1 * xs + xli * i);
				yp1 = super.scene2d.yToPixel(y1 + vor1 * ys + yli * i);
				xp2 = super.scene2d.xToPixel(x1 + vor2 * xs + xli* (i + 1));
				yp2 = super.scene2d.yToPixel(y1 + vor2 * ys + yli * (i + 1));
			
				g2.drawLine(xp1, yp1, xp2, yp2);
			}
			
			xp1 = super.scene2d.xToPixel(x1 - xs + 2 * sizeZ * xli);
			yp1 = super.scene2d.yToPixel(y1 - ys + 2 * sizeZ * yli);
			xp2 = super.scene2d.xToPixel(x2);
			yp2 = super.scene2d.yToPixel(y2);
			
			g2.drawLine(xp1, yp1, xp2, yp2);
		}
		
	/*	double xp1 = super.scene2d.xToPixel(x1);
		double yp1 = super.scene2d.yToPixel(y1);
		double xp2 = super.scene2d.xToPixel(x2);
		double yp2 = super.scene2d.yToPixel(y2);

		int breite = sizeX;
		int dicke = sizeY;
		double xli, yli, xs, ys, vor1, vor2;

		g2.setColor(color);

		// Setzen der Dicke der Feder
		g2.setStroke(new BasicStroke((dicke >= 0) ? dicke : -dicke));
		if (!(xp1 == xp2 & yp1 == yp2)) {
			// Zeichnen der Feder
			xli = ((xp2 - xp1) / (2 * sizeZ + 1));
			yli = ((yp2 - yp1) / (2 * sizeZ + 1));
			xs = (breite * (-yli) / Maths.hypot((xli), (yli)));
			ys = (breite * (xli) / Maths.hypot((xli), (yli)));

			g2.drawLine((int) xp1, (int) yp1, (int) (xp1 + xs + xli),
					(int) (yp1 + ys + yli));

			for (int i = 1; i < 2 * sizeZ; i++) {

				vor1 = (int) Math.pow((-1), i + 1);
				vor2 = (int) Math.pow((-1), i);
				g2.drawLine((int) (xp1 + vor1 * xs + xli * i), (int) (yp1
						+ vor1 * ys + yli * i), (int) (xp1 + vor2 * xs + xli
						* (i + 1)), (int) (yp1 + vor2 * ys + yli * (i + 1)));
			}
			g2.drawLine((int) (xp1 - xs + 2 * sizeZ * xli), (int) (yp1 - ys + 2
					* sizeZ * yli), (int) xp2, (int) yp2);
		}*/
	}
}
