package uibk.mtk.parser.realparser;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
 * Klasse zum Zerlegen der Eingabe in Token
 * 
 * 
 * <p>
 * Diese Klasse ist dem frei verwendbaren Packet <i>expr </i> von Darius Bacon
 * (siehe http://www.accesscom.com/~darius/) entnommen und wurde ge?ndert.
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
class Scanner {

	/** die Eingabe */
	private String inputstring;

	/** erlaubte Operatoren */
	private String operatorChars;

	/** bekannte Variablen */
	private Hashtable variables;

	/** der Vektor mit den Token */
	Vector tokens = new Vector();

	/** Position im Eingabe-String */
	int index = -1;

	public Scanner(String string, String operatorChars, Hashtable variables) {

		this.inputstring = string;
		this.variables = variables;
		this.operatorChars = operatorChars;

		//Zerlegung des Eingabe Strings
		int i = 0;
		do {
			i = scanToken(i);
		} while (i < inputstring.length());

		addAsterisk();

	}

	/**
	 * Gibt zur?ck ob ein Token eine Variable ist, d.h. ob
	 *  
	 */
	private boolean isVariable(Token t) {
		boolean isVar = false;

		if (variables.size() > 0) {
			String var = null;
			for (Enumeration e = variables.elements(); e.hasMoreElements();) {
				var = ((Variable) e.nextElement()).toString();
				if (t.sval.equals(var))
					isVar = true;
			}
		}

		return isVar;
	}

	/**
	 * Einsetzten bei: "Nummer" + "(" "Variable" + "(" "Variable" + "Wort"
	 * "Nummer" + "Wort" "Wort" +"Nummer" ")" + "(" ")" + "Wort" ")" + "Nummer"
	 *  
	 */
	private void addAsterisk() {
		for (int i = 0; i < tokens.size() - 1; i++) {
			Token tokenleft = (Token) tokens.elementAt(i);
			Token tokenright = (Token) tokens.elementAt(i + 1);

			if (tokenleft.ttype == Token.TT_NUMBER) {
				if (tokenright.ttype == '('
						|| tokenright.ttype == Token.TT_WORD)
					tokens.add(i + 1, new Token('*', 0, "*", 0, 1)); //$NON-NLS-1$
			}

			if (tokenleft.ttype == ')') {
				if (tokenright.ttype == '('
						|| tokenright.ttype == Token.TT_WORD
						|| tokenright.ttype == Token.TT_NUMBER)
					tokens.add(i + 1, new Token('*', 0, "*", 0, 1)); //$NON-NLS-1$

			}

			if (isVariable(tokenleft)) {
				// if (tokenright.ttype == '(' || tokenright.ttype ==
				// Token.TT_WORD ||
				//   tokenright.ttype == Token.TT_NUMBER ||
				// isVariable(tokenright))
				if (tokenright.ttype == '(')
					tokens.add(i + 1, new Token('*', 0, "*", 0, 1)); //$NON-NLS-1$
			}
		}
	}

	/** Gibt die Eingabe zur?ck */
	public String getInput() {
		return inputstring;
	}

	/** Gibt den String mit den erlaubten Operatoren zur?ck */
	public String getOperatorChars() {
		return operatorChars;
	}

	/** gibt <CODE>true</CODE> zur?ck, falls kein Token vorhanden sind */
	public boolean isEmpty() {
		return tokens.size() == 0;
	}

	public boolean atStart() {
		return index <= 0;
	}

	public boolean atEnd() {
		return tokens.size() <= index;
	}

	public Token nextToken() {
		++index;
		return getCurrentToken();
	}

	public Token previousToken() {
		--index;
		return getCurrentToken();
	}

	public Token getCurrentToken() {
		if (atEnd() || index < 0)
			return new Token(Token.TT_EOF, 0, inputstring,
					inputstring.length(), inputstring.length());
		return (Token) tokens.elementAt(index);
	}

	private int scanToken(int i) {
		//?ber whitespace Symbole hinweggehen
		while (i < inputstring.length()
				&& Character.isWhitespace(inputstring.charAt(i)))
			++i;

		if (i == inputstring.length()) {
			return i;
		} else if (0 <= operatorChars.indexOf(inputstring.charAt(i))) {
			//falls der aktuelle Buchstabe ein Operator ist
			if (i + 1 < inputstring.length()) {
				//falls m?glich n?chsten Buchstaben mitnehemn und vgl.
				String pair = inputstring.substring(i, i + 2);
				int ttype = 0;
				if (pair.equals("<=")) //$NON-NLS-1$
					ttype = Token.TT_LE;
				else if (pair.equals(">=")) //$NON-NLS-1$
					ttype = Token.TT_GE;
				else if (pair.equals("<>"))ttype = Token.TT_NE; //$NON-NLS-1$

				if (0 != ttype) {//wir haben schon einen Token gefunden
					tokens
							.addElement(new Token(ttype, 0, inputstring, i,
									i + 2));
					return i + 2;
				}
			}
			//
			tokens.addElement(new Token(inputstring.charAt(i), 0, inputstring,
					i, i + 1));
			return i + 1;
		} else if (Character.isLetter(inputstring.charAt(i))) {
			//falls Buchstabe ->Wort scannen
			return scanSymbol(i);
		} else if (Character.isDigit(inputstring.charAt(i))
				|| '.' == inputstring.charAt(i)) {
			//falls Zahl bzw. Zahlzeichen -> Zahl scannen
			return scanNumber(i);
		} else {
			//Fehlertoken erzeugen
			tokens.addElement(makeErrorToken(i, i + 1));
			return i + 1;
		}
	}

	private int scanSymbol(int i) {

		//zuerst pr?fen ob registrierte Variable beginnt
		//nehmen sie aber nur dazu wenn danach kein Buchstabe oder Zahl
		//->damit wir richtige Fehlermeldung unbekannte var. bekommen

		if (variables.size() > 0) {
			Variable var = null;
			for (Enumeration e = variables.elements(); e.hasMoreElements();) {
				var = (Variable) e.nextElement();
				String v = var.toString();
				if (i + v.length() <= inputstring.length())
					if (inputstring.substring(i, i + v.length()).equals(v)) {

						//Pr?fen ob dieses schon hinzugef?gt
						boolean addToken = true;
						//String temp=s.substring(i,i+v.length());
						for (int k = 0; k < tokens.size(); k++)
							if (((Token) tokens.elementAt(k)).sval
									.equals(inputstring))
								addToken = false;

						//	danach darf kein Buchstabe kommen
						if ((i + v.length()) < inputstring.length()
								&& (Character.isLetter(inputstring.charAt(i
										+ v.length())) || Character
										.isDigit(inputstring.charAt(i
												+ v.length()))))
							addToken = false;

						if (addToken) {
							tokens.addElement(new Token(Token.TT_WORD, 0,
									inputstring, i, i + v.length()));
							return i + v.length();
						}

					}

			}
		}

		int from = i;

		while (i < inputstring.length()
				&& (Character.isLetter(inputstring.charAt(i)) || Character
						.isDigit(inputstring.charAt(i))))
			++i;
		tokens.addElement(new Token(Token.TT_WORD, 0, inputstring, from, i));
		return i;
	}

	private int scanNumber(int i) {
		int from = i;

		for (; i < inputstring.length(); ++i)
			if ('.' != inputstring.charAt(i)
					&& !Character.isDigit(inputstring.charAt(i))
					&& inputstring.charAt(i) != 'E'
					&& !inputstring.substring(i - 1, i + 1).equals("E-")) //$NON-NLS-1$
				break;

		String text = inputstring.substring(from, i);
		double nval;
		try {
			nval = Double.valueOf(text).doubleValue();
		} catch (NumberFormatException nfe) {
			tokens.addElement(makeErrorToken(from, i));
			return i;
		}

		tokens
				.addElement(new Token(Token.TT_NUMBER, nval, inputstring, from,
						i));
		return i;
	}

	/** Erzeugen eines FehlerTokens */
	private Token makeErrorToken(int from, int i) {
		return new Token(Token.TT_ERROR, 0, inputstring, from, i);
	}
}