package uibk.mtk.swing.appletbase;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.JApplet;
import javax.swing.JFrame;

/**
 * Frame der bei den Applets verwendet wird, falls sie als eigenst?ndige
 * Applikation laufen.
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */

class AppletFrame extends JFrame {

	/** Applet das in diesem Fenster laufen soll */
	JApplet applet;

	/** minimale Breite des Fensters */
	int minimumwidth;

	/** minimale H?he des Fensters */
	int minimumheight;

	class ResizeListener extends ComponentAdapter {

		/**
		 * Wir verhindern eine Verkleinerung des Frames unter die durch
		 * minimumwidth und minimumheight festgeleget Gr??e verhindert.
		 */
		public void componentResized(ComponentEvent event) {
			int width = getWidth();
			int height = getHeight();
			if (width < minimumwidth) {
				setSize(minimumwidth, height);
				return;
			}
			if (height < minimumheight) {
				setSize(width, minimumheight);
				return;
			}

			applet.repaint();
		}

	}

	/**
	 * Erzeugt das Fenster mit der ?bergebenen Breite und H?he.
	 * 
	 * @param applet
	 *            Applet das in diesem Frame ausgef?hrt werden soll
	 * @param width
	 *            Ausgangsbreite des Fensters
	 * @param height
	 *            Ausgangsh?he H?he des Fensters
	 * @param minimumWidth
	 *            minimale Breite des Fensters
	 * @param minimumHeight
	 *            minimale H?he des Fensters
	 */
	public AppletFrame(JApplet applet, int width, int height, int minimumWidth,
			int minimumHeight) {

		setSize(width, height);

		this.applet = applet;
		this.minimumheight = minimumHeight;
		this.minimumwidth = minimumWidth;

		//Zentrieren des Fensters in der Mitte des Bildschirms
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

		setLocation((screenSize.width - getSize().width) / 2,
				(screenSize.height - getSize().height) / 2);

		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		addComponentListener(new ResizeListener());

		//Applet initialisieren und Starten
		applet.init();
		applet.start();

		//Applet zum Frame hinzuf?gen
		getContentPane().add(applet, BorderLayout.CENTER);

		//sichtbar machen
		setVisible(true);
	}

}