package uibk.mtk.swing.appletbase;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;

import uibk.mtk.lang.Messages;
import uibk.mtk.swing.base.ButtonNoFocus;

/**
 * Navigationsleiste f?r alle Applets.
 * 
 * <p>
 * Die Navigationsleiste enth?lt 4 Buttons mit Icons. Die ersten 3 ?ffnen die
 * URLs "index.html#help","index.html#theory" und "index.html" (dies
 * funktioniert nur falls die Applets in einem Browser ausgef?hrt werden,
 * deshalb sind diese drei Kn?pfe falls das Applet als eigenst?ndige Applikation
 * ausgef?hrt wird, nicht sichtbar (sie werden mittels <CODE>
 * enableInternetButtons</CODE> bei der Initialisierung von <CODE>AppletBase
 * </CODE> deaktiviert). Der vierte Knopf gibt schlie?lich eine kurze
 * Copyrightinformation aus.
 * 
 * </p>
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class NavigationBar extends JToolBar implements ActionListener {

	/** Button f?r Copyrightinformation */
	protected ButtonNoFocus buttonInfo;

	/** Button zum ?ffene von "index.html" */
	protected ButtonNoFocus buttonHome;

	/** Button zum ?ffene von "index.html#theory" */
	protected ButtonNoFocus buttonTheory;

	/** Button zum ?ffene von "index.html#help" */
	protected ButtonNoFocus buttonHelp;

	/** das Applet das diese Navigationsleiste verwendet */
	AppletBase applet;

	/** Label in das die Copyrightinfo ausgegeben wird */
	JLabel labelReport;

	/** Copyright-Information */
	protected String strcopyright = Messages.getString(BUNDLE_NAME,
			"NavigationBar.0"); //$NON-NLS-1$

	protected String strauthor = Messages.getString(BUNDLE_NAME,
			"NavigationBar.1"); //$NON-NLS-1$
	
	protected String home = "../index.html";

	/** Label f?r dem Knopf f?r die Information */
	private String labelinfo = Messages.getString(BUNDLE_NAME,
			"NavigationBar.2"); //$NON-NLS-1$

	/** Label f?r dem Knopf f?r die Homepage */
	private String labelhome = Messages.getString(BUNDLE_NAME,
			"NavigationBar.3"); //$NON-NLS-1$

	/** Label f?r dem Knopf f?r die Hilfe */
	private String labelhelp = Messages.getString(BUNDLE_NAME,
			"NavigationBar.4"); //$NON-NLS-1$

	/** Label f?r dem Knopf f?r die Theory */
	private String labeltheory = Messages.getString(BUNDLE_NAME,
			"NavigationBar.5"); //$NON-NLS-1$

	/** Name des Resourcebundles */
	private static final String BUNDLE_NAME = "uibk.mtk.swing.appletbase.messages";//$NON-NLS-1$

	/**
	 * erzeugt die Navigationsleiste mit den ?bergebenen Strings als
	 * Beschriftung
	 * 
	 * @param applet
	 *            das Applet das diese Navigationsleiste verwendet
	 */
	public NavigationBar(AppletBase applet, String info, String home,
			String help, String theory) {

		labelinfo = info;
		labelhelp = help;
		labeltheory = theory;
		labelhome = home;
		this.applet = applet;
		initComponents();
	}

	/**
	 * erzeugt die Navigationsleiste
	 * 
	 * @param applet
	 *            das Applet das diese Navigationsleiste verwendet
	 */
	public NavigationBar(AppletBase applet) {
		this.applet = applet;
		initComponents();
	}

	/**
	 * Erzeugt einen Button mit den angegebenen Bild,Text,Tooltip und
	 * ActionCommand
	 */
	ButtonNoFocus createButton(String stringurl, String text, String tooltip,
			String actioncommand) {
		URL url = this.getClass().getResource(stringurl);

		ButtonNoFocus button = new ButtonNoFocus(text, new ImageIcon(url));

		if (text != null && !text.equals("")) { //$NON-NLS-1$
			button.setVerticalTextPosition(SwingConstants.BOTTOM);
			button.setHorizontalTextPosition(SwingConstants.CENTER);
			button.setFont(new Font("Arial", Font.PLAIN, 10)); //$NON-NLS-1$
		}

		button.setToolTipText(tooltip);

		button.setActionCommand(actioncommand);

		button.addActionListener(this);

		return button;
	}

	private void initComponents() {

		setFloatable(false);

		this.setLayout(new FlowLayout(FlowLayout.RIGHT, 20, 2));
		this.setMinimumSize(new Dimension(400, 150));

		setBorder(BorderFactory.createLineBorder(Color.black));

		buttonInfo = createButton(
				"/uibk/mtk/swing/res/question.gif", labelinfo, "Info", "info"); //$NON-NLS-1$       
		buttonHome = createButton(
				"/uibk/mtk/swing/res/house.gif", labelhome, "Homepage", "home"); //$NON-NLS-1$
		buttonTheory = createButton(
				"/uibk/mtk/swing/res/blackboard.gif", labeltheory, "Theorie", "theory"); //$NON-NLS-1$ 
		buttonHelp = createButton(
				"/uibk/mtk/swing/res/help.gif", labelhelp, "Hilfe", "help"); //$NON-NLS-1$ 

		this.add(buttonHelp);
		this.add(buttonTheory);
		this.add(buttonHome);
		this.add(buttonInfo);

	}

	/** setzt das Label in das die Copyrightinfo ausgegeben wird */
	public void setReportingLabel(JLabel label) {
		this.labelReport = label;
	}

	/** Ereignisverarbeitung: Laden der URLs */
	public void actionPerformed(ActionEvent ev) {

		if (ev.getActionCommand().equals("info")) { //$NON-NLS-1$
			if (labelReport != null) {
				labelReport.setForeground(Color.black);
				labelReport.setText(strcopyright);
			}
			System.out.println(strcopyright);
			System.out.println(strauthor);

		}
		if (ev.getActionCommand().equals("home")) {
			try {

				applet.getAppletContext()
						.showDocument(
								new URL(applet.getDocumentBase(),
										home), "_blank");
			} catch (MalformedURLException ex) {
			}
		}
		if (ev.getActionCommand().equals("help")) {

			try {
				applet.getAppletContext()
						.showDocument(
								new URL(applet.getDocumentBase(),
										"index.html#help"), "_blank");
			} catch (MalformedURLException ex) {
			}
		}
		if (ev.getActionCommand().equals("theory")) {

			try {
				applet.getAppletContext()
						.showDocument(
								new URL(applet.getDocumentBase(),
										"index.html#theory"), "_blank");
			} catch (MalformedURLException ex) {
			}
		}

	}

	/**
	 * aktiviert/deaktiviert (d.h macht sie sichtbar/unsichtbar) die 3 Buttons
	 * Help/Home/Theory die eine Internetseite ?ffnen, was aber f?r eine
	 * Applikation (die nicht als Applet im Webbrowser l?uft), nicht m?glich
	 * ist.
	 */
	public void enableInternetButtons(boolean enable) {

		buttonHelp.setVisible(enable);
		buttonHome.setVisible(enable);
		buttonTheory.setVisible(enable);
	}
	
	/**
	 * aktiviert/deaktiviert (d.h macht sie sichtbar/unsichtbar) den Theory Button
	 */
	public void enableTheoryButton(boolean enable) {

		buttonTheory.setVisible(enable);
	}
	
	/** Setzt den Text des Autors */
	public void setStrauthor(String text) {
		strauthor = text;
	}
	/** Setzt den Text fr Copyright */
	public void setStrcopyright(String text) {
		strcopyright = text;
	}
	/** Setzt das "home" der Homepage */
	public void setHome(String text) {
		home = text;
	}
	
	/** Setzt den Tect f?r den "Hilfe"-Knopf */
	public void setTextHelp(String text) {
		buttonHelp.setText(text);
		buttonHelp.setToolTipText(text);
	}

	/** Setzt den Text f?r den "Info"-Knopf */
	public void setTextInfo(String text) {
		buttonInfo.setText(text);
		buttonInfo.setToolTipText(text);
	}

	/** Setzt den Text f?r den "Theorie"-Knopf */
	public void setTextTheory(String text) {
		buttonTheory.setText(text);
		buttonTheory.setToolTipText(text);
	}

	/** Setzt den Text f?r den "Home"-Knopf */
	public void setTextHome(String text) {
		buttonHome.setText(text);
		buttonHome.setToolTipText(text);
	}

	/** Setzt den Text f?r Copyright-Ausgabe */
	public void setTextCopyright(String text) {
		strcopyright = text;

	}
}