package uibk.mtk.swing.appletbase;

import java.awt.Color;

import javax.swing.JLabel;

import uibk.mtk.lang.Messages;
import uibk.mtk.lang.Reporter;

/**
 * Label f?r die Ausgabe der (Fehler)-Meldungen
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class ReportingLabel extends JLabel implements Reporter {

	/** Name des Resourcebundles */
	private static final String BUNDLE_NAME = "uibk.mtk.swing.appletbase.messages";//$NON-NLS-1$

	/** Gibt eine normale Meldung (in schwarz) in das Label aus */
	public void report(String message) {
		setForeground(Color.black);
		setText(message);
	}

	/** Erzeugt eine leeres Label */
	public ReportingLabel() {
		super();
	}

	/** Erzeugt das Label mit dem ?bergebeben Text */
	public ReportingLabel(String text) {
		super(text);

	}

	/**
	 * Gibt die Message dieser Fehlermeldung im Label aus
	 *  
	 */
	public void reportError(Exception ex) {
		//f?r leichte Fehlersuche
		//ex.printStackTrace();
		reportError(ex.getMessage());
	}

	/**
	 * Gibt eine Fehlermeldung (in rot) in das Label aus
	 * 
	 * @param title
	 *            Titel der Fehlermeldung
	 * @param ex
	 *            Exception deren message ausgegben wird
	 */
	public void reportError(String title, Exception ex) {
		//f?r leichte Fehlersuche
		//ex.printStackTrace();
		reportError(title + ": " + ex.getMessage()); //$NON-NLS-1$
	}

	/** Gibt eine Fehlermeldung (in rot) in das Label aus */
	public void reportError(String message) {
		if (message == null || message.equals("")) //$NON-NLS-1$
			message = Messages.getString(BUNDLE_NAME, "ReportingLabel.2"); //$NON-NLS-1$
		setForeground(new Color(255, 51, 0));
		setText(message);

	}

	/** Gibt eine Erfolgsmeldung (in gr?n) in das Label aus */
	public void reportSuccess(String message) {
		setForeground(new Color(0, 200, 0));
		setText(message);

	}

}