package uibk.mtk.swing.base;

import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;

/**
 * Klasse f?r einen Swing-Button der nicht im Fokus sein kann und auch den Fokus
 * nicht zeichnet. Falls der Button mit einem Bild erzeugt wird, wird
 * setBorderPainted(false) gesetzt. (auf setBorder(null) wird verzichtet, da das
 * neu XP Look and Feel immer die Border zeichnet (siehe bug id: 4881191) und
 * bei setBorder(null) der Button schlecht aussieht)
 * 
 * 
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class ButtonNoFocus extends JButton {

	/** Setzen der gew?nschten Eigenschaften des Knopfes */
	private void init() {

		setFocusPainted(false);
		setBorderPainted(false);
		setBorder(BorderFactory.createEmptyBorder());
	}

	/** Erzeugt einen nicht-fokusierbaren Knopf mit Bild */
	public ButtonNoFocus(ImageIcon icon) {

		super(icon);
		init();

	}

	/***************************************************************************
	 * @param icon
	 *            Bild
	 * @param actioncommand
	 *            actioncommand f?r den Button
	 * @param action
	 *            <CODE>ActionListener</CODE> f?r den Button
	 * @param tooltip
	 *            <CODE>Tooltip</CODE>
	 */
	public ButtonNoFocus(ImageIcon icon, String actioncommand,
			ActionListener action, String tooltip) {

		super(icon);
		init();

		setActionCommand(actioncommand);
		addActionListener(action);
		setToolTipText(tooltip);
	}

	/** Erzeugt einen nicht-fokusierbaren Knopf mit Text und Bild */
	public ButtonNoFocus(String text, ImageIcon icon) {
		super(text, icon);
		init();

	}

	/** Erzeugt einen nicht-fokusierbaren Knopf mit angegebenen Text */
	public ButtonNoFocus(String text) {
		super(text);
		init();
	}

	/** zur?ckgeben das Knopf nicht fokusierbar ist */
	public boolean isFocusable() {
		return false;
	}

	/** zur?ckgeben das Knopf nicht fokusierbar ist */
	public boolean setRequestFocusEnabled() {
		return false;
	}

}