package uibk.mtk.swing.scene2d;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JRadioButton;

import uibk.mtk.draw2d.base.MathPanel2D;
import uibk.mtk.draw2d.base.Scene2D;
import uibk.mtk.geom.geom2d.CoordinateRect2D;
import uibk.mtk.lang.Messages;
import uibk.mtk.swing.base.ButtonNoFocus;
import uibk.mtk.swing.base.TitledPanel;

/**
 * Panel zum Steuern der Skalierung f?r ein zweidimensionales Koordinatensystem.
 * Die Skalierung in den x- un y-Richtung kann entweder unabh?ngig oder abh?ngig
 * von einander vorgenommen werden. Wenn zw. diesen beiden Optionen umgeschalten
 * wird, werden wieder die OrginalKoordinaten ?bernommen (diese werden mit
 * </CODE> update</CODE> gesetzt)
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class PanelScale2D extends TitledPanel implements ActionListener {

	/** Option "gleich skalieren" */
	protected JRadioButton optAxesEqual;

	/** Option "unabh?ngig skalieren" */
	protected JRadioButton optAxesIndependent;

	/** Knopf zum Verkleinern der Skalierung in beide Richtung */
	protected ButtonNoFocus buttonBothZoomIn;

	/** Knopf zum Vergr??ern der Skalierung in beide Richtung */
	protected ButtonNoFocus buttonBothZoomOut;

	/** Knopf zum Verkleinern der Skalierung in x-Richtung */
	protected ButtonNoFocus buttonXZoomIn;

	/** Knopf zum Vergr??ern der Skalierung x-Richtung */
	protected ButtonNoFocus buttonXZoomOut;

	/** Knopf zum Verkleinern der Skalierung in y-Richtung */
	protected ButtonNoFocus buttonYZoomIn;

	/** Knopf zum Vergr??ern der Skalierung in y-Richtung */
	protected ButtonNoFocus buttonYZoomOut;

	/**
	 * MathPanel3D der die <CODE>Scene2D</CODE> enth?lt deren Skalierung
	 * ge?ndert werden soll
	 */
	MathPanel2D mathpanel2d;

	/** Die Grenzen beim letzten Aufruf von <CODE>update()</CODE> */
	CoordinateRect2D limits = new CoordinateRect2D();

	/** Name des Resourcebundles */
	private static final String BUNDLE_NAME = "uibk.mtk.swing.scene2d.messages";//$NON-NLS-1$

	private static final String STRTITLE = Messages.getString(BUNDLE_NAME,
			"PanelScale2D.0"); //$NON-NLS-1$

	private static final String STRINDEPENDENTLY = Messages.getString(
			BUNDLE_NAME, "PanelScale2D.1"); //$NON-NLS-1$

	private static final String STREQUAL = Messages.getString(BUNDLE_NAME,
			"PanelScale2D.2"); //$NON-NLS-1$

	/** Sklarierungsfaktor */
	private double scalefactor = 1.3;

	/** Szene die skaliert wird */
	Scene2D scene2d;

	/** Erzeugt den Panel mit den Steuerelemente zum steuern der Optionen */
	public PanelScale2D(MathPanel2D mathpanel2d) {
		super(STRTITLE);
		if (mathpanel2d == null)
			throw new NullPointerException("MathPanel is null"); //$NON-NLS-1$
		this.mathpanel2d = mathpanel2d;
		scene2d = mathpanel2d.getScene2d();
		initComponents();
	}

	/** Initialisieren der Koponenten */
	void initComponents() {

		//Layout
		setLayout(new GridBagLayout());
		setMaximumSize(new Dimension(2000, 220));

		//Erzeugen der Optionskn?pfe
		optAxesIndependent = new JRadioButton(STRINDEPENDENTLY);
		optAxesIndependent.setActionCommand("axes_independent"); //$NON-NLS-1$
		optAxesIndependent.addActionListener(this);

		optAxesEqual = new JRadioButton(STREQUAL);
		optAxesEqual.setActionCommand("axes_equal"); //$NON-NLS-1$
		optAxesEqual.addActionListener(this);

		ButtonGroup groupAxesOpt = new ButtonGroup();
		groupAxesOpt.add(optAxesEqual);
		groupAxesOpt.add(optAxesIndependent);

		//Erzugen der Kn?pfe zum Skalieren

		URL urlIconPlus = this.getClass().getResource("/uibk/mtk/swing/res/plus.gif"); //$NON-NLS-1$
		URL urlIconMinus = this.getClass().getResource("/uibk/mtk/swing/res/minus.gif"); //$NON-NLS-1$
		buttonBothZoomIn = new ButtonNoFocus(new ImageIcon(urlIconPlus));
		buttonBothZoomOut = new ButtonNoFocus(new ImageIcon(urlIconMinus));

		buttonBothZoomIn.setActionCommand("bothzoomin"); //$NON-NLS-1$
		buttonBothZoomOut.setActionCommand("bothzoomout"); //$NON-NLS-1$

		buttonXZoomIn = new ButtonNoFocus(new ImageIcon(urlIconPlus));
		buttonXZoomOut = new ButtonNoFocus(new ImageIcon(urlIconMinus));
		buttonXZoomIn.setActionCommand("xzoomin"); //$NON-NLS-1$
		buttonXZoomOut.setActionCommand("xzoomout"); //$NON-NLS-1$

		buttonYZoomIn = new ButtonNoFocus(new ImageIcon(urlIconPlus));
		buttonYZoomOut = new ButtonNoFocus(new ImageIcon(urlIconMinus));
		buttonYZoomIn.setActionCommand("yzoomin"); //$NON-NLS-1$
		buttonYZoomOut.setActionCommand("yzoomout"); //$NON-NLS-1$

		buttonBothZoomIn.addActionListener(this);
		buttonBothZoomOut.addActionListener(this);
		buttonXZoomIn.addActionListener(this);
		buttonXZoomOut.addActionListener(this);
		buttonYZoomIn.addActionListener(this);
		buttonYZoomOut.addActionListener(this);

		//richtige Option aktivieren
		update();

		//Komponenten hinzuf?gen
		add(optAxesEqual, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));
		add(this.buttonBothZoomOut, new GridBagConstraints(1, 0, 1, 1, 0.0,
				0.0, GridBagConstraints.WEST, GridBagConstraints.NONE,
				new Insets(0, 0, 0, 0), 0, 0));
		add(this.buttonBothZoomIn, new GridBagConstraints(2, 0, 1, 1, 0.00,
				0.0, GridBagConstraints.WEST, GridBagConstraints.NONE,
				new Insets(0, 10, 0, 0), 0, 0));

		add(buttonYZoomIn, new GridBagConstraints(1, 1, 2, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						10, 15, 0, 0), 0, 0));
		add(optAxesIndependent, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));
		add(buttonXZoomOut, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));
		add(buttonXZoomIn, new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						10, 0, 0), 0, 0));
		add(buttonYZoomOut, new GridBagConstraints(1, 3, 2, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						15, 0, 0), 0, 0));
	}

	/** Ereignisverarbeitung */
	public void actionPerformed(ActionEvent ev) {

		//in beide Richtungen gr??er
		if (ev.getActionCommand().equals("bothzoomout")) { //$NON-NLS-1$

			scene2d.setScaleFactorX(scene2d.getScaleFactorX() * scalefactor);
			scene2d.setScaleFactorY(scene2d.getScaleFactorY() * scalefactor);

		}

		//in beide Richtungen kleiner
		if (ev.getActionCommand().equals("bothzoomin")) { //$NON-NLS-1$

			scene2d
					.setScaleFactorX(scene2d.getScaleFactorX() * 1
							/ scalefactor);
			scene2d
					.setScaleFactorY(scene2d.getScaleFactorY() * 1
							/ scalefactor);

		}

		//in x-Richtung gr??er
		if (ev.getActionCommand().equals("xzoomout")) { //$NON-NLS-1$

			scene2d.setScaleFactorX(scene2d.getScaleFactorX() * scalefactor);

		}

		//in y-Richtung gr??er
		if (ev.getActionCommand().equals("yzoomout")) { //$NON-NLS-1$
			scene2d.setScaleFactorY(scene2d.getScaleFactorY() * scalefactor);

		}

		//in x-Richtung kleiner
		if (ev.getActionCommand().equals("xzoomin")) { //$NON-NLS-1$

			scene2d
					.setScaleFactorX(scene2d.getScaleFactorX() * 1
							/ scalefactor);

		}

		//in y-Richtung kleiner
		if (ev.getActionCommand().equals("yzoomin")) { //$NON-NLS-1$

			scene2d
					.setScaleFactorY(scene2d.getScaleFactorY() * 1
							/ scalefactor);

		}

		//auf gemeinsames Skalarieren gehen
		if (ev.getActionCommand().equals("axes_equal")) { //$NON-NLS-1$
			scene2d.setLimits(limits);
			scene2d.axesequal(true);
			scene2d.setScaleFactorX(1);
			scene2d.setScaleFactorY(1);

		}

		//auf unabh?ngiges Skalarieren gehen
		if (ev.getActionCommand().equals("axes_independent")) { //$NON-NLS-1$
			scene2d.setLimits(limits);
			scene2d.axesequal(false);
			scene2d.setScaleFactorX(1);
			scene2d.setScaleFactorY(1);

		}

		update();
		mathpanel2d.repaint();

	}

	/**
	 * Aktiviert abh?ngig von der gew?hlten Skalierungsoption die entsprechenden
	 * Komponenten des Panels und deaktivert die anderen und speichert die
	 * aktuellen Koordinaten
	 */
	public void update() {

		limits = (CoordinateRect2D) mathpanel2d.getScene2d()
				.getOriginalLimits().clone();
		//aktivieren der richtigen Komponenten
		if (mathpanel2d.getScene2d().isAxesEqualEnabled())
			optAxesEqual.setSelected(true);
		else
			optAxesIndependent.setSelected(true);

		if (optAxesEqual.isSelected()) {
			optAxesEqual.setSelected(true);
			buttonXZoomIn.setEnabled(false);
			buttonXZoomOut.setEnabled(false);
			buttonYZoomIn.setEnabled(false);
			buttonYZoomOut.setEnabled(false);
			buttonBothZoomIn.setEnabled(true);
			buttonBothZoomOut.setEnabled(true);

		} else {
			optAxesIndependent.setSelected(true);

			buttonXZoomIn.setEnabled(true);
			buttonXZoomOut.setEnabled(true);
			buttonYZoomIn.setEnabled(true);
			buttonYZoomOut.setEnabled(true);
			buttonBothZoomIn.setEnabled(false);
			buttonBothZoomOut.setEnabled(false);
			optAxesIndependent.setSelected(true);

		}
	}

	/**
	 * Setzt den Skalierungsfaktor
	 * 
	 * @param factor
	 *            >1
	 */
	public void setScaleFactor(double factor) {
		if (factor <= 1)
			throw new IllegalArgumentException(""); //$NON-NLS-1$
		this.scalefactor = factor;
	}

	/** (de)aktiviert den Panel und die Steuerelemente */
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);

		if (enabled) {

			if (mathpanel2d.getScene2d().isAxesEqualEnabled())
				optAxesEqual.setSelected(true);
			else
				optAxesIndependent.setSelected(true);

			update();
		}

	}

}